//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Fond.h"
#include "Lineo.h"

Fond::Fond(Lineo *app_lineo, Uint8 type, SDL_Rect contour, Uint32 z_index)
{
	_AppLineo = app_lineo;
	_Type = type;
	_Contour = contour;
    _ZIndex = z_index;

    _ImgName=DATA_PATH;
    switch(_Type){
        case FOND_MENU:
            _ImgName.append("menus/fond_menu.png");
            _ImgW      = 800;
            _ImgH      = 600;
            _ImgMargeG = 200;
            _ImgMargeD =  0;
            _ImgMargeH =  41;
            _ImgMargeB =  42;
        break;
        case FOND_SAISIE:
            _ImgName.append("menus/fond_saisie.png");
            _ImgW     = 602;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB = 10;
        break;
        case FOND_TABLEAU:
            _ImgName.append("menus/fond_tableau.png");
            _ImgW     = 640;
            _ImgH     = 480;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  20;
        break;
        case FOND_TABLEAU_LIGNE:
            _ImgName.append("menus/fond_tableau_ligne.png");
            _ImgW     = 640;
            _ImgH     = 32;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  8;
        break;
        case FOND_POPUP_ERREUR:
            _ImgName.append("popup/fond_error.png");
            _ImgW     = 800;
            _ImgH     = 600;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  44;
        break;
        case FOND_POPUP_ERREUR_BTN_ON:
            _ImgName.append("popup/fond_error_bouton_on.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
        case FOND_POPUP_ERREUR_BTN_OUT:
            _ImgName.append("popup/fond_error_bouton_out.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
        case FOND_POPUP_WARNING:
            _ImgName.append("popup/fond_warning.png");
            _ImgW     = 800;
            _ImgH     = 600;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  44;
        break;
        case FOND_POPUP_WARNING_BTN_ON:
            _ImgName.append("popup/fond_warning_bouton_on.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
        case FOND_POPUP_WARNING_BTN_OUT:
            _ImgName.append("popup/fond_warning_bouton_out.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
        case FOND_POPUP_INFO:
            _ImgName.append("popup/fond_info.png");
            _ImgW     = 800;
            _ImgH     = 600;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  44;
        break;
        case FOND_POPUP_INFO_BTN_ON:
            _ImgName.append("popup/fond_info_bouton_on.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
        case FOND_POPUP_INFO_BTN_OUT:
            _ImgName.append("popup/fond_info_bouton_out.png");
            _ImgW     = 800;
            _ImgH     =  38;
            _ImgMargeG = _ImgMargeD = _ImgMargeH = _ImgMargeB =  15;
        break;
    }

	static Uint32 unique_cpt=0;
    _SpriteName = _ImgName + to_string(unique_cpt++);

    init();
    draw();
}

Fond::~Fond()
{
	_AppLineo->_Screen->delSprite(_SpriteName);
}

// initialise les rectangles src et dst
void Fond::init()
{
    // on initialise tous les rectangles sources
    _HautGaucheRect[FOND_RECT_SRC].x=0;
    _HautGaucheRect[FOND_RECT_SRC].y=0;
    _HautGaucheRect[FOND_RECT_SRC].w=_ImgMargeG;
    _HautGaucheRect[FOND_RECT_SRC].h=_ImgMargeH;

    _HautRect[FOND_RECT_SRC].x=_ImgMargeG;
    _HautRect[FOND_RECT_SRC].y=0;
    _HautRect[FOND_RECT_SRC].w=_Contour.w-_ImgMargeG-_ImgMargeD;
    _HautRect[FOND_RECT_SRC].h=_ImgMargeH;

    _HautDroitRect[FOND_RECT_SRC].x=_ImgW-_ImgMargeD;
    _HautDroitRect[FOND_RECT_SRC].y=0;
    _HautDroitRect[FOND_RECT_SRC].w=_ImgMargeD;
    _HautDroitRect[FOND_RECT_SRC].h=_ImgMargeH;

    _DroitRect[FOND_RECT_SRC].x=_ImgW-_ImgMargeD;
    _DroitRect[FOND_RECT_SRC].y=_ImgMargeH;
    _DroitRect[FOND_RECT_SRC].w=_ImgMargeD;
    _DroitRect[FOND_RECT_SRC].h=_Contour.h-_ImgMargeH-_ImgMargeB;

    _BasDroitRect[FOND_RECT_SRC].x=_ImgW-_ImgMargeD;
    _BasDroitRect[FOND_RECT_SRC].y=_ImgH-_ImgMargeB;
    _BasDroitRect[FOND_RECT_SRC].w=_ImgMargeD;
    _BasDroitRect[FOND_RECT_SRC].h=_ImgMargeB;

    _BasRect[FOND_RECT_SRC].x=_ImgMargeG;
    _BasRect[FOND_RECT_SRC].y=_ImgH-_ImgMargeB;
    _BasRect[FOND_RECT_SRC].w=_Contour.w-_ImgMargeG-_ImgMargeD;
    _BasRect[FOND_RECT_SRC].h=_ImgMargeB;

    _BasGaucheRect[FOND_RECT_SRC].x=0;
    _BasGaucheRect[FOND_RECT_SRC].y=_ImgH-_ImgMargeB;
    _BasGaucheRect[FOND_RECT_SRC].w=_ImgMargeG;
    _BasGaucheRect[FOND_RECT_SRC].h=_ImgMargeB;

    _GaucheRect[FOND_RECT_SRC].x=0;
    _GaucheRect[FOND_RECT_SRC].y=_ImgMargeH;
    _GaucheRect[FOND_RECT_SRC].w=_ImgMargeG;
    _GaucheRect[FOND_RECT_SRC].h=_Contour.h-_ImgMargeH-_ImgMargeB;

    _CentreRect[FOND_RECT_SRC].x=_ImgMargeG;
    _CentreRect[FOND_RECT_SRC].y=_ImgMargeH;
    _CentreRect[FOND_RECT_SRC].w=_Contour.w-_ImgMargeG-_ImgMargeD;
    _CentreRect[FOND_RECT_SRC].h=_Contour.h-_ImgMargeH-_ImgMargeB;

    // on calcul tous les positions destination
    // juste lees positions car la sdl ne s'occupe pas des dimensions
    // sur les destinations
    _HautGaucheRect[FOND_RECT_DST].x=_Contour.x;
    _HautGaucheRect[FOND_RECT_DST].y=_Contour.y;

    _HautRect[FOND_RECT_DST].x=_Contour.x+_ImgMargeG;
    _HautRect[FOND_RECT_DST].y=_Contour.y;

    _HautDroitRect[FOND_RECT_DST].x=_Contour.x+_Contour.w-_ImgMargeD;
    _HautDroitRect[FOND_RECT_DST].y=_Contour.y;

    _DroitRect[FOND_RECT_DST].x=_Contour.x+_Contour.w-_ImgMargeD;
    _DroitRect[FOND_RECT_DST].y=_Contour.y+_ImgMargeH;

    _BasDroitRect[FOND_RECT_DST].x=_Contour.x+_Contour.w-_ImgMargeD;
    _BasDroitRect[FOND_RECT_DST].y=_Contour.y+_Contour.h-_ImgMargeB;

    _BasRect[FOND_RECT_DST].x=_Contour.x+_ImgMargeG;
    _BasRect[FOND_RECT_DST].y=_Contour.y+_Contour.h-_ImgMargeB;

    _BasGaucheRect[FOND_RECT_DST].x=_Contour.x;
    _BasGaucheRect[FOND_RECT_DST].y=_Contour.y+_Contour.h-_ImgMargeB;

    _GaucheRect[FOND_RECT_DST].x=_Contour.x;
    _GaucheRect[FOND_RECT_DST].y=_Contour.y+_ImgMargeH;

    _CentreRect[FOND_RECT_DST].x=_Contour.x+_ImgMargeG;
    _CentreRect[FOND_RECT_DST].y=_Contour.y+_ImgMargeH;
}

// affiche le Fond  l'cran
void Fond::draw(void)
{
    _AppLineo->_Screen->addSprite(_SpriteName,_ImgName,_ZIndex,true);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_HautGaucheRect[FOND_RECT_SRC],_HautGaucheRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_HautRect[FOND_RECT_SRC],_HautRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_HautDroitRect[FOND_RECT_SRC],_HautDroitRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_DroitRect[FOND_RECT_SRC],_DroitRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_BasDroitRect[FOND_RECT_SRC],_BasDroitRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_BasRect[FOND_RECT_SRC],_BasRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_BasGaucheRect[FOND_RECT_SRC],_BasGaucheRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_GaucheRect[FOND_RECT_SRC],_GaucheRect[FOND_RECT_DST]);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_CentreRect[FOND_RECT_SRC],_CentreRect[FOND_RECT_DST]);
}

// dplace le fond suivant un offset
void Fond::mov(Sint32 offset_x, Sint32 offset_y)
{
	_AppLineo->_Screen->movSpriteAllImages(_SpriteName, offset_x, offset_y);
}

void Fond::setZIndex(Uint32 z_index)
{
    _AppLineo->_Screen->setSpriteZIndex(_SpriteName,z_index);
}
