//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "GamePlateau.h"
#include "Lineo.h"

GamePlateau::GamePlateau(Lineo* app_lineo)
{
	_AppLineo = app_lineo;
    SDL_Rect rec_src,rec_des;
    // on initialise tous les pointeurs de classes  NULL
    for(Uint8 i=0;i<16;i++){
        _AutourPieces[i]=NULL;
    }
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            _PlateauPieces[i][j]=NULL;
        }
    }
    _ChoixPieces[GP_CHOIX_LEFT]=NULL;
    _ChoixPieces[GP_CHOIX_RIGHT]=NULL;
    // on initialise l'angle de vue du plateau
    _PlateauAngle=string_to_int(_AppLineo->_Config->get("plateau_angle"));
    _PlateauAngleMove=0.0;
    // on initialise les 16 coordonnes autour du plateau
    calculAllAutourPoints();
    // on initialise les variables concernants le survol des pices autour
    _AutourSurvolPiece=false;
    _AutourPiecesI=0;
    // on initialise les coordonnes du choix
    calculAllChoixPoints();
    // on initialise les 16 coordonnes d'origine du plateau pour l'angle rgl au dessus
    // mais galement le poin de la boule
    calculAllPlateauPoints();
    // on initialise les variables concernants le survol des cercles
    _PlateauSurvolCercle=false;
    _PlateauCercleI=0;
    _PlateauCercleJ=0;
    // on indique qu'on est pas en mode tourner le plateau
    _PlateauTourneOn=false;
    // on init les variables concernant la gestion ou non des vnements pices et cercles
    _CercleWait=false;
    _PieceWait=false;
    // on dit que tous les cercles sont libres et on nomme les sprites des cercles en mme temps
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            _PlateauCerclesLibre[i][j] = true;
            _PlateauCerclesName[i][j] = "cercle_" + to_string(i) + "_" + to_string(j);
        }
    }
    // puis le plateau vierge de tout cercle
    _AppLineo->_Screen->addSprite("plateau",(string)(DATA_PATH)+"game/plateau.png",Z_PLATEAU);
    rec_src.x=0;
    rec_des.x=GP_PLATEAU_X;
    rec_src.y=0;
    rec_des.y=GP_PLATEAU_Y;
    rec_src.w=rec_des.w=GP_PLATEAU_W;
    rec_src.h=rec_des.h=GP_PLATEAU_H;
    _AppLineo->_Screen->addSpriteImage("plateau",rec_src,rec_des);
    // puis la boule du plateau
    rec_src.x=0;
    rec_src.y=0;
    rec_src.w=GP_BOULE_W;
    rec_src.h=GP_BOULE_H;
    rec_des.w=GP_BOULE_W;
    rec_des.h=GP_BOULE_H;
    rec_des.x=_PlateauBoulePoint.x-GP_BOULE_W_DIV_2;
    rec_des.y=_PlateauBoulePoint.y-GP_BOULE_H_DIV_2;
    _PlateauBouleRect=rec_des; // on mmorise le rectangle pour pouvoir bouger le cercle et calculer sont offset
    _AppLineo->_Screen->addSprite("boule",(string)(DATA_PATH)+"game/boule.png",Z_PLATEAU+5);
    _AppLineo->_Screen->addSpriteImage("boule",rec_src,rec_des);
    // ensuite les cercles
    rec_src.x=0;
    rec_src.y=0;
    rec_src.w=GP_CERCLE_W;
    rec_src.h=GP_CERCLE_H;
    rec_des.w=GP_CERCLE_W;
    rec_des.h=GP_CERCLE_H;
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            rec_src.x=0;
            rec_des.x=_PlateauPoints[i][j].x-GP_CERCLE_W_DIV_2;
            rec_des.y=_PlateauPoints[i][j].y-GP_CERCLE_H_DIV_2;
            _PlateauCerclesRect[i][j]=rec_des; // on mmorise le rectangle pour pouvoir bouger le cercle et calculer sont offset
            _AppLineo->_Screen->addSprite(_PlateauCerclesName[i][j],(string)(DATA_PATH)+"game/cercle.png",Z_PLATEAU+10);
            _AppLineo->_Screen->addSpriteImage(_PlateauCerclesName[i][j],rec_src,rec_des);
            // on ajoute l'image de slection
            rec_src.x=82;
            _AppLineo->_Screen->addSpriteImage(_PlateauCerclesName[i][j],rec_src,rec_des);
            // et on positionne sur la premire image
            _AppLineo->_Screen->setSpriteCurrent(_PlateauCerclesName[i][j],0);
        }
    }
    // puis les pices autour du plateau
    for(Uint8 i=0;i<16;i++){
        if(_AutourPieces[i]!=NULL){
            _AutourPieces[i]->drawScreen(_AutourPoints[i].x,_AutourPoints[i].y,Z_PLATEAU+_AutourPoints[i].y);
        }
    }
    // puis les pices du plateau
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            if(_PlateauPieces[i][j]!=NULL){
                _PlateauPieces[i][j]->drawScreen(_PlateauPoints[i][j].x,_PlateauPoints[i][j].y,Z_PLATEAU+_PlateauPoints[i][j].y);
            }
        }
    }
}

GamePlateau::~GamePlateau()
{
    // on efface le plateau
    _AppLineo->_Screen->delSprite("plateau");
    // puis les cercles
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            _AppLineo->_Screen->delSprite(_PlateauCerclesName[i][j]);
        }
    }
    // on efface la boule
    _AppLineo->_Screen->delSprite("boule");
    // on supprime toutes les classes
    for(Uint8 i=0;i<16;i++){
        delete(_AutourPieces[i]);
    }
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            delete(_PlateauPieces[i][j]);
        }
    }
    delete(_ChoixPieces[GP_CHOIX_LEFT]);
    delete(_ChoixPieces[GP_CHOIX_RIGHT]);
    // on affiche le curseur flche
    _AppLineo->_Curseur->setCurseur(CUR_FLECHE);
}

// repositionne les objets  l'cran suivant les nouveaux points calculs
void GamePlateau::updateScreen(void)
{
    SDL_Rect old_rect;
    Sint32 offset_x;
    Sint32 offset_y;

    // on recalcul tous les point suivant le nouvel _PlateauAngle
    calculAllPlateauPoints();

    // on rafraichit ensuite l'emplacement des cercles
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            old_rect=_PlateauCerclesRect[i][j];
            _PlateauCerclesRect[i][j].x=_PlateauPoints[i][j].x-GP_CERCLE_W_DIV_2;
            _PlateauCerclesRect[i][j].y=_PlateauPoints[i][j].y-GP_CERCLE_H_DIV_2;
            offset_x = _PlateauCerclesRect[i][j].x - old_rect.x;
            offset_y = _PlateauCerclesRect[i][j].y - old_rect.y;
            _AppLineo->_Screen->movSpriteAllImages(_PlateauCerclesName[i][j],offset_x,offset_y);
        }
    }

    // puis on rafraichit l'emplacement de la boule
    old_rect=_PlateauBouleRect;
    _PlateauBouleRect.x=_PlateauBoulePoint.x-GP_BOULE_W_DIV_2;
    _PlateauBouleRect.y=_PlateauBoulePoint.y-GP_BOULE_H_DIV_2;
    offset_x = _PlateauBouleRect.x - old_rect.x;
    offset_y = _PlateauBouleRect.y - old_rect.y;
    _AppLineo->_Screen->movSpriteAllImages("boule",offset_x,offset_y);

    // puis celui des pices
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            if(_PlateauPieces[i][j]!=NULL){
                _PlateauPieces[i][j]->updateScreen(_PlateauPoints[i][j].x,_PlateauPoints[i][j].y,Z_PLATEAU+_PlateauPoints[i][j].y);
            }
        }
    }

}

// gre les vnements de la vue
void GamePlateau::processEvents(SDL_Event *event)
{
    // on s'occupe de notre vue en premier
    switch (event->type) {
        case SDL_USEREVENT:
            switch (event->user.code) {
            }
        break;
        case SDL_MOUSEBUTTONDOWN:
            if(event->button.button==SDL_BUTTON_LEFT){
                // on vrifi si on survol le plateau
                if(_SurvolPlateau){
                    // on vrifie ensuite que l'on ne soit sur aucun cercle libre si on est en attente
                    bool test=false;
                    if(_CercleWait){
                        for(Uint8 i=0;i<4;i++){
                            for(Uint8 j=0;j<4;j++){
                                if(_PlateauCerclesLibre[i][j] && testSurvolCercle(i,j,event->motion.x,event->motion.y)){
                                    test=true;
                                }
                            }
                        }
                    }
                    if(!test){
                        // on est sur aucun cercle donc on peut se mettre en mode tourner le plateau
                        _PlateauTourneOn=true;
                    }
                }
            }
        break;
        case SDL_MOUSEBUTTONUP:
            if(event->button.button==SDL_BUTTON_LEFT){
                // on indique qu'on est plus en mode tourner le plateau qu'on y tait ou non avant
                _PlateauTourneOn=false;
                // on vrifie si on survolait une pice, car si c'est le cas il faut qu'on l'envoi  la vue game
                // elle sera quoi en faire :-)
                if(_AutourSurvolPiece){
                    _AppLineo->_Events->addEvent(EVENT_GAME_PIECE_SELECTED,&_AutourPiecesI);
                }
                // on vrifie si on survolait un cercle, car si c'est le cas il faut qu'on l'envoi  la vue game
                // elle sera quoi en faire :-)
                if(_PlateauSurvolCercle){
                    _AppLineo->_Events->addEvent(EVENT_GAME_CERCLE_SELECTED,&_PlateauCercleI,&_PlateauCercleJ);
                }
            }
        break;
        case SDL_MOUSEMOTION:
            // on met  jour la variable _SurvolPlateau
            _SurvolPlateau = testSurvolPlateau(event->motion.x,event->motion.y);
            // on vrifi si on est en mode tourner le plateau, si c'est le cas on bouge le plateau suivant le dplacement de la souris
            if(_PlateauTourneOn){ // **** MODE TOURNER LE PLATEAU ON ****
                const double x = (double)(event->motion.x - GP_OFFSET_X);
                const double y = (double)(event->motion.y - GP_OFFSET_Y);
                const double x_old = (double)(x + event->motion.xrel);
                const double y_old = (double)(y + event->motion.yrel);
                // on calcul l'angle de la souris actuel par rapport  l'axe x
                const double angle1 = rad2deg(atan(y/x));
                // on calcul l'angle de la souris prcdent
                const double angle2 = rad2deg(atan(y_old/x_old));
                // on calcul l'angle entre le point actuel de la souris et l'ancien
                _PlateauAngleMove += angle2 - angle1;
                // si on dpasse un seuil dans un sens ou dans l'autre on redessine la vue suivant le nouvel angle
                if(_PlateauAngleMove < -5.0 || _PlateauAngleMove > 5.0){
                    // ces deux conditions vites les dbordements quand _PlateauAngleMove est tout d'un coup = (+/-)170
                    if(_PlateauAngleMove<-50.0)_PlateauAngleMove=+10.0;
                    if(_PlateauAngleMove>50.0)_PlateauAngleMove=-10.0;
                    // on recalcul l'angle du plateau
                    _PlateauAngle+=(Sint16)_PlateauAngleMove;
                    if(_PlateauAngle<0)_PlateauAngle+=360;
                    if(_PlateauAngle>359)_PlateauAngle-=360;
                    // on enregistre la nouvelle valeur de l'angle dans le fichier config
					_AppLineo->_Config->set("plateau_angle", to_string(_PlateauAngle));
                    // et on update la vue
                    updateScreen();
                    _PlateauAngleMove=0.0;
                }
            }else{ // **** MODE TOURNER LE PLATEAU OFF ****
                // on dtecte si on survole une pice
                _AutourSurvolPiece=false;
                if(_PieceWait){
                    Uint32 y_max=0; // correspond au z en quelque sorte, on y enregistrera la valeur y de la pice survole qui le plus grand y
                    for(Uint8 i=0;i<16;i++){
                        if(_AutourPieces[i]!=NULL && y_max<_AutourPoints[i].y){
                            if(_AutourPieces[i]->testSurvol(event->motion.x,event->motion.y)){
                                y_max = _AutourPoints[i].y;
                                _AutourSurvolPiece=true;
                                _AutourPiecesI=i;
                            }
                        }
                    }
                    // on en profite pour mettre un contour ou l'enlever suivant le cas
                    for(Uint8 i=0;i<16;i++){
                        if(_AutourPieces[i]!=NULL){
                            if(_AutourPiecesI == i && _AutourSurvolPiece == true){
                                _AutourPieces[i]->setSelected(true);
                            }else{
                                _AutourPieces[i]->setSelected(false);
                            }
                        }
                    }
                }
                // on dtecte si on survol un cercle
                _PlateauSurvolCercle=false;
                if(_CercleWait){
                    for(Uint8 i=0;i<4;i++){
                        for(Uint8 j=0;j<4;j++){
                            if(_PlateauCerclesLibre[i][j] && testSurvolCercle(i,j,event->motion.x,event->motion.y)){
                                _PlateauSurvolCercle=true;
                                _PlateauCercleI = i;
                                _PlateauCercleJ = j;
                                // on met le cercle en slection
                                _AppLineo->_Screen->setSpriteCurrent(_PlateauCerclesName[i][j],1);
                            }else{
                                // on enlve la slection du cercle
                                _AppLineo->_Screen->setSpriteCurrent(_PlateauCerclesName[i][j],0);
                            }
                        }
                    }
                }
            }
        break;
   	}
   	// on met  jour le curseur en focntion des infos que nous avons
   	setCurseur();
}

// calcul les 16 points autour du plateau o se trouverons les pices en attendant d'tre sur le plateau
void GamePlateau::calculAllAutourPoints(void)
{
    Uint16 angle;
    Uint16 R,r;

    R=287;
    r=164;
    angle=12;

    for(Uint8 i=0;i<16;i++){
       _AutourPoints[i] = calculPoint(angle*i+180,R,r);
    }
}

// calcul des 2 points de choix  gauche et  droite devant le plateau
void GamePlateau::calculAllChoixPoints(void)
{
    Uint16 R,r;

    R=336;
    r=194;

    _ChoixPoints[GP_CHOIX_LEFT] = calculPoint(135,R,r);
    _ChoixPoints[GP_CHOIX_RIGHT] = calculPoint(45,R,r);
}

// recalcul les points suivant _PlateauAngle et repositionne les objets
//  l'cran en consquence
// >>> concrtement il y a 3 llipses qui contiennent chacune plusieurs points
// +++ et il ya aussi le point de la "Boule"
void GamePlateau::calculAllPlateauPoints(void)
{
    Uint16 angle;
    Uint16 R,r;
    // on s'occupe de la premire llipse qui englobe les 4 points du centre
    R=59;
    r=34;
    // point_1_1
    angle=_PlateauAngle+135;
    _PlateauPoints[1][2]=calculPoint(angle,R,r);
    // point_1_2
    angle=_PlateauAngle+225;
    _PlateauPoints[1][1]=calculPoint(angle,R,r);
    // point_2_1
    angle=_PlateauAngle+45;
    _PlateauPoints[2][2]=calculPoint(angle,R,r);
    // point_2_2
    angle=_PlateauAngle+315;
    _PlateauPoints[2][1]=calculPoint(angle,R,r);
    // puis la deuxime llipse qui englobe 8 points
    R=134;
    r=78;
    // point_3_1
    angle=_PlateauAngle+18;
    _PlateauPoints[3][2]=calculPoint(angle,R,r);
    // point_2_0
    angle=_PlateauAngle+72;
    _PlateauPoints[2][3]=calculPoint(angle,R,r);
    // point_1_0
    angle=_PlateauAngle+108;
    _PlateauPoints[1][3]=calculPoint(angle,R,r);
    // point_0_1
    angle=_PlateauAngle+162;
    _PlateauPoints[0][2]=calculPoint(angle,R,r);
    // point_0_2
    angle=_PlateauAngle+198;
    _PlateauPoints[0][1]=calculPoint(angle,R,r);
    // point_1_3
    angle=_PlateauAngle+252;
    _PlateauPoints[1][0]=calculPoint(angle,R,r);
    // point_2_3
    angle=_PlateauAngle+288;
    _PlateauPoints[2][0]=calculPoint(angle,R,r);
    // point_3_2
    angle=_PlateauAngle+342;
    _PlateauPoints[3][1]=calculPoint(angle,R,r);
    // puis la troisime lipse qui englobe les 4 points de l'extrmit
    R=177;
    r=102;
    // point_0_0
    angle=_PlateauAngle+135;
    _PlateauPoints[0][3]=calculPoint(angle,R,r);
    // point_0_3
    angle=_PlateauAngle+225;
    _PlateauPoints[0][0]=calculPoint(angle,R,r);
    // point_3_0
    angle=_PlateauAngle+45;
    _PlateauPoints[3][3]=calculPoint(angle,R,r);
    // point_3_3
    angle=_PlateauAngle+315;
    _PlateauPoints[3][0]=calculPoint(angle,R,r);

    // puis du point de la boule
    R=200;
    r=115;
    angle=_PlateauAngle+90;
    _PlateauBoulePoint = calculPoint(angle,R,r);
}

// calcul les coordonnes x et y d'un point d'aprs un angle et R et r qui sont les rayons d'une lipse
// on considre le centre du plateau comme rayon commun  toutes les lipses
point2D GamePlateau::calculPoint(Uint16 angle, Uint16 R, Uint16 r)
{
    if(angle > 359) angle-=360;

    point2D pt;
/*
#ifdef Nokia770
    pt.x = (COS[angle] * R)+GP_OFFSET_X;
    pt.y = (SIN[angle] * r)+GP_OFFSET_Y;
#else
*/
    pt.x = (Uint16)(COS[angle] * R)+(Uint16)(GP_OFFSET_X);
    pt.y = (Uint16)(SIN[angle] * r)+(Uint16)(GP_OFFSET_Y);
//#endif
    return pt;
}

// ajoute une pice autour du plateau
void GamePlateau::addAutourPiece(Uint8 i, Uint8 forme)
{
    if(i<16){
        if(_AutourPieces[i]!=NULL){
            delAutourPiece(i);
        }
        // on cr la pice
        _AutourPieces[i]= new Piece(_AppLineo,forme);
        // puis on l'affiche dans Screen
        _AutourPieces[i]->drawScreen(_AutourPoints[i].x,_AutourPoints[i].y,Z_PLATEAU+_AutourPoints[i].y);
    }
}

// supprime une pice autour du plateau
void GamePlateau::delAutourPiece(Uint8 i)
{
    if(i<16 && _AutourPieces[i]!=NULL){
        if(_AutourSurvolPiece==true && _AutourPiecesI==i){
            _AutourSurvolPiece=false;
        }
        delete(_AutourPieces[i]);
        _AutourPieces[i]=NULL;
    }
}

// ajoute une pice devant le plateau
void GamePlateau::addChoixPiece(Uint8 i_autour, bool gauche)
{
    if(i_autour<16){
        Uint8 i_choix = (gauche == 0 ? 0 : 1);
        if(_ChoixPieces[i_choix]!=NULL){
            delChoixPiece(i_choix);
        }
        // on cr la pice
        _ChoixPieces[i_choix]= new Piece(_AppLineo, _AutourPieces[i_autour]->getForme());
        // puis on l'affiche dans Screen
        _ChoixPieces[i_choix]->drawScreen(_ChoixPoints[i_choix].x,_ChoixPoints[i_choix].y,Z_PLATEAU+_ChoixPoints[i_choix].y);
    }
}

// supprime une pice devant le plateau
void GamePlateau::delChoixPiece(Uint8 i)
{
    if(i<2 && _ChoixPieces[i]!=NULL){
        delete(_ChoixPieces[i]);
        _ChoixPieces[i]=NULL;
    }
}

// ajoute une pice sur le plateau
void GamePlateau::addPlateauPiece(Uint8 i, Uint8 j, Uint8 forme)
{
    if(i<4 && j<4){
        if(_PlateauPieces[i][j]!=NULL){
            delPlateauPiece(i,j);
        }
        // on cr la pice
        _PlateauPieces[i][j]= new Piece(_AppLineo, forme);
        // puis on l'affiche dans Screen
        _PlateauPieces[i][j]->drawScreen(_PlateauPoints[i][j].x,_PlateauPoints[i][j].y,Z_PLATEAU+_PlateauPoints[i][j].y);
        // puis on fait le ncessaire pour que le cercle ne soit plus slectionn
        _PlateauSurvolCercle=false;
        _AppLineo->_Screen->setSpriteCurrent(_PlateauCerclesName[i][j],0);
        _PlateauCerclesLibre[i][j]=false;
    }
}

// supprime une pice sur le plateau
void GamePlateau::delPlateauPiece(Uint8 i, Uint8 j)
{
    if(i<4 && j<4 && _PlateauPieces[i][j]!=NULL){
        delete(_PlateauPieces[i][j]);
        _PlateauPieces[i][j]=NULL;
        // on prcise que le cercle est  nouveau libre
        _PlateauCerclesLibre[i][j]=true;
    }
}

// dtecte si on survol le plateau (true) ou non (false)
bool GamePlateau::testSurvolPlateau(Uint16 x_mouse, Uint16 y_mouse)
{
    // on dcale les coordonnes de la souris sur l'angle en haut  gauche du plateau
    Sint16 x = x_mouse - GP_PLATEAU_X;
    Sint16 y = y_mouse - GP_PLATEAU_Y;
    // on vrifie d'abord qu'on est bien sur la surface
    if(x>=0 && x<GP_PLATEAU_W && y>=0 && y<GP_PLATEAU_H){
        // puis qu'on est bien sur le plateau en vrifiant la transparence qui doit tre  255 (opaque) pour tre sr
        if(_AppLineo->_Screen->getSpriteSurfacePixelAlpha("plateau",x,y)==SDL_ALPHA_OPAQUE){
            return true;
        }
    }
    return false;
}

// dtecte si on survol le cercle[i][j] (true) ou non (false)
bool GamePlateau::testSurvolCercle(Uint8 i, Uint8 j, Uint16 x_mouse, Uint16 y_mouse)
{
    // on dcale les coordonnes de la souris sur l'angle en haut  gauche du cercle
    Uint16 cercle_x = _PlateauPoints[i][j].x-GP_CERCLE_W_DIV_2;
    Uint16 cercle_y = _PlateauPoints[i][j].y-GP_CERCLE_H_DIV_2;
    Sint16 x = x_mouse - cercle_x;
    Sint16 y = y_mouse - cercle_y;
    // on vrifie d'abord qu'on est bien sur la surface
    if(x>=0 && x<GP_CERCLE_W && y>=0 && y<GP_CERCLE_H){
        // puis qu'on est bien sur le cercle en vrifiant la transparence qui doit tre  255 (opaque) pour tre sr
        if(_AppLineo->_Screen->getSpriteSurfacePixelAlpha(_PlateauCerclesName[i][j],x,y)==SDL_ALPHA_OPAQUE){
            return true;
        }
    }
    return false;
}

// surligne un cercle et sa pice afin d'indiquer qu'elles appartiennent  un Lineo
void GamePlateau::setLineo(Uint8 i, Uint8 j)
{
    // on met le cercle en slection
    _AppLineo->_Screen->setSpriteCurrent(_PlateauCerclesName[i][j],1);
    // puis on entoure la pice
    if(_PlateauPieces[i][j]!=NULL){
        _PlateauPieces[i][j]->setSelected(true);
    }
}

// permet de dfinir le curseur  afficher en fonction de tous les paramtres du jeu
void GamePlateau::setCurseur(void)
{
    Uint8 curseur;

    if(_HumanPlay){
        curseur = CUR_FLECHE;
    }else{
        curseur = CUR_FLECHE_DORT;
    }

    if(_PlateauTourneOn){
       curseur = CUR_TOURNE;
    }else{
        if(_AutourSurvolPiece || _PlateauSurvolCercle){
            curseur = CUR_MAIN;
        }else if(!_PlateauSurvolCercle && _SurvolPlateau){
            curseur=CUR_FLECHE_TOURNE;
        }
    }

    _AppLineo->_Curseur->setCurseur(curseur);
}
