//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Joueur.h"
#include "Lineo.h"

Joueur::Joueur(Lineo* app_lineo, Uint8 type, string nom, Uint8 place)
{
    init(app_lineo, type, true, nom, place);
}

Joueur::Joueur(Lineo* app_lineo, Uint8 type, Uint8 place)
{
    init(app_lineo, type, false, "", place);
}

Joueur::~Joueur()
{
	delete(_JoueurIA);
#ifndef Nokia770
	delete(_JoueurNet);
#endif
}

// initialisation de la classe directement appele par le constructeur
void Joueur::init(Lineo* app_lineo, Uint8 type, bool set_nom, string nom, Uint8 place)
{
   	_AppLineo   = app_lineo;
    _Type       = type;
    _Place      = place;
    _NomIsSet   = set_nom;
    if(_NomIsSet){
        _Nom    = nom;
    }
	_JoueurIA   = NULL;
#ifndef Nokia770
    _JoueurNet  = NULL;
#endif
    switch(_Type){
        case JOUEUR_TYPE_IA:
            _JoueurIA = new JoueurIA(_AppLineo);
        break;
#ifndef Nokia770
        case JOUEUR_TYPE_RESO:
            _JoueurNet = new JoueurNet(_AppLineo,this);
        break;
#endif
    }
}

// on demande au joueur de choisir la premire pice de la partie
void Joueur::pieceFirstWait(VueGame *vue_game, LaDonne donne)
{
    switch(_Type){
        case JOUEUR_TYPE_IA:
            _JoueurIA->pieceFirstWait(donne);
        break;
#ifndef Nokia770
        case JOUEUR_TYPE_RESO:
            _JoueurNet->pieceWait(vue_game);
        break;
#endif
    }
}

// on demande au joueur de choisir une pice
void Joueur::pieceWait(VueGame *vue_game, LaDonne donne)
{
    switch(_Type){
        case JOUEUR_TYPE_IA:
            _JoueurIA->pieceWait(donne);
        break;
#ifndef Nokia770
        case JOUEUR_TYPE_RESO:
            _JoueurNet->pieceWait(vue_game);
        break;
#endif
    }
}

// on demande au joueur de choisir un emplacement sur le plateau pour la pice choisie
void Joueur::cercleWait(VueGame *vue_game, LaDonne donne)
{
    switch(_Type){
        case JOUEUR_TYPE_IA:
            _JoueurIA->cercleWait(donne);
        break;
#ifndef Nokia770
        case JOUEUR_TYPE_RESO:
            _JoueurNet->cercleWait(vue_game);
        break;
#endif
    }
}

// choix du joueur qui commence
Uint8 Joueur::getQuiCommence()
{
#ifndef Nokia770
    if(_Type == JOUEUR_TYPE_RESO) {
        return _JoueurNet->getQuiCommence();
    }else{
#endif
        // on initialise le gnrateur alatoire
        srand(SDL_GetTicks());
        // puis on choisit un nombre alatoire
        Uint8 max_rand = 2;
        return rand() % max_rand;
#ifndef Nokia770
    }
#endif
}

// initialise le nom du joueur
void Joueur::setNom(string nom)
{
    _NomIsSet   = true;
    _Nom        = nom;
}

// appel les fonctions de traitement des events des classes IA et reseau selon le type du joueur initialis
void Joueur::processEvents(SDL_Event *event)
{
    switch(_Type){
#ifndef Nokia770
        case JOUEUR_TYPE_RESO:
            _JoueurNet->processEvents(event);
        break;
#endif
    }
}
