//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "JoueurIA.h"
#include "Lineo.h"

JoueurIA::JoueurIA(Lineo* app_lineo)
{
	_AppLineo = app_lineo;
    // on rcupre le niveau qui est dans le .ini
    _Niveau = _AppLineo->_Config->get("niveau");
}

JoueurIA::~JoueurIA()
{

}

// choisi alatoirement la premire pice de la partie
void JoueurIA::pieceFirstWait(LaDonne donne)
{
    // on initialise le gnrateur alatoire
    srand(SDL_GetTicks());
    // puis on choisit un nombre alatoire
    Uint16 max_rand = 16;
    Uint16 i = rand() % max_rand;
    // on termine en choisissant le coup  faire
    _PieceChoisie = i;
    // on donne la pice que l'on a choisit
	_AppLineo->waitUntilPopUpIsNotNull();
    _AppLineo->_Events->addEvent(EVENT_GAME_PIECE_SELECTED,&_PieceChoisie);
}

void JoueurIA::pieceWait(LaDonne donne)
{
    // on donne la pice que l'on a choisit
    _AppLineo->waitUntilPopUpIsNotNull();
    _AppLineo->_Events->addEvent(EVENT_GAME_PIECE_SELECTED,&_PieceChoisie);
}

void JoueurIA::cercleWait(LaDonne donne)
{
    // on cherche le meilleur coup
    meilleurCoup(donne);
    // on place la pice qui nous est donne sur le cercle choisit ci-avant
    _AppLineo->waitUntilPopUpIsNotNull();
    _AppLineo->_Events->addEvent(EVENT_GAME_CERCLE_SELECTED,&_IChoisi,&_JChoisi);
}

// dtermine le meilleur coup  jouer pour l'IA suivant les diffrents
// critres rgls dans le constructeur
void JoueurIA::meilleurCoup(LaDonne donne)
{
    // on cr le noeud reprsentant la partie actuelle
    NoeudIA first_noeud(donne);
    if(donne.nbLibres() == 0){
        dernierCoup(first_noeud);
    // si le plateau  0 ou une pice dessus on joue au hasard, pour :
    //  - viter des calculs
    //  - a n'a pas d'importance
    //  - impossible pour l'adversaire de trouver une stratgie dans le premier coup ;-)
    }else if(donne.nbLibres() > 13){
        noeudHasard(first_noeud);
    // sinon on recherche le meilleur noeuds fils amenant  la victoire
    // ou du moins au meilleur rsultat possible
    }else{
        strategie(first_noeud);
    }
}

// stratgie qui dtermine le meilleur coup  choisir suivant le niveau de difficult
// si il ne trouve pas un seul meilleur il cr une liste et cherche alatoirement dedans
// ==> ceci permet de ne pas toujours donner la premire pice de la liste et de
// placer sur la premire case libre trouve
// *****************************************************************************
//                           S T R A T E G I E
//
// ==> on fait toujours le minimum vers le maximum suivant le niveau choisi
// ex : nivau difficile, on ferra facile, puis moyen, puis difficile pour finir
//
// ==> Modifier le 4 sept 05 : on dcale les I.A.
//
// Facile :
//      - on test si on peut faire un Lineo
//
// Moyen :
//      - on test si on peut faire un Lineo
//      - on vrifi que la pice qu'on donne ne fasse pas un Lineo
//
// Difficile :
//      - on cherche le coup qui nous permmettra de faire un Lineo avec la
//      piece que nous donnera ventuellement l'adversaire aprs qu'on est jou
//
// Impossible :
//      - on accepte les Lineo en carr ce qui rajoute 9 possibilits d'o la
//      difficult accru
//
// Mode GOD ou compt ??? : pas cod pour le moment
//      - on verifit qu'on ne fait pas deux tierces opposes sur un mme critre
//      - on essaye de faire une tierce
//      - on essaye de donner une piece qui permet de faire deux tierces opposes
//      sur un mme critre
//      - on essaye de donner une pice qui permet de faire une tierce
//
// *****************************************************************************
void JoueurIA::strategie(NoeudIA noeud)
{
    // on cr la liste des coup possible pour l'IA
    // on dmarre avec cette liste
    vnoeud liste_IA = listeSousNoeuds(noeud);

    // on amliore la liste des coups en fonction du niveau
    if(_Niveau == "facile"){
        liste_IA = listeFacile(liste_IA);
    }else if(_Niveau == "moyen"){
        liste_IA = listeMoyen(liste_IA);
    }else if(_Niveau == "difficile"){
        liste_IA = listeDifficile(liste_IA);
    }else if(_Niveau == "impossible"){
        liste_IA = listeImpossible(liste_IA);
    }

	//****************************************************************
    // aprs avoir pur la liste_IA on choisit un coup au hasard
    // ce coup est plus ou moins sr suivant le succs des diffrents
    // filtres prcdents
	//****************************************************************
    // on initialise le gnrateur alatoire
    srand(SDL_GetTicks());
    // puis on choisit un nombre alatoire
    Uint16 max_rand = liste_IA.size();
    Uint16 x = rand() % max_rand;
    // on termine en choisissant le coup  faire
    _IChoisi = liste_IA[x].getNextI();
    _JChoisi = liste_IA[x].getNextJ();
    _PieceChoisie = liste_IA[x].getNextPieceChoisie();
}

// calcul la meilleur liste de coup possible en mode facile
vnoeud JoueurIA::listeFacile(vnoeud liste_IA)
{
    vnoeud new_liste;
    Uint8 size_IA = liste_IA.size();
    // on parcour la liste et on vrifi si on a fait un Lineo
    for(Uint8 i=0;i<size_IA;i++){
        if(liste_IA[i].isLineo()){
            // on quitte dj en mettant un seul coup dans la liste car il est gagnant
            new_liste.push_back(liste_IA[i]);
            return new_liste;

        }
    }
    // sinon on retourne la liste_IA tel quel car on ne peut rien faire dessus
    return liste_IA;
}

// calcul la meilleur liste de coup possible en mode moyen
vnoeud JoueurIA::listeMoyen(vnoeud liste_IA)
{
    vnoeud new_liste;
    Uint8 size_IA = liste_IA.size();
    // on parcour la liste et on vrifi si on a fait un Lineo
    // on garde tous les bons lments dans une liste  part
    for(Uint8 i=0;i<size_IA;i++){
        if(liste_IA[i].isLineo()){
            // on quitte dj en mettant un seul coup dans la liste car il est gagnant
            new_liste.push_back(liste_IA[i]);
            return new_liste;

        }
    }
    // on cr une liste des coup pour chaque coup que l'on fait et on vrifi que
    // a ne donne pas un Lineo  l'adversaire
    new_liste.clear();
    for(Uint8 i=0;i<size_IA;i++){
        // ce delay permet de ne pas monopoliser le processeur pour cecci
        SDL_Delay(IA_SDL_DELAY);

        vnoeud liste_adv = listeSousNoeuds(liste_IA[i]);
        Uint8 size_adv = liste_adv.size();
        bool find_lineo=false;
        for(Uint8 j=0;j<size_adv && !find_lineo;j++){
            if(liste_adv[j].isLineo()){
                find_lineo = true;
            }
        }
        // on ajoute ce coup dans la liste temp si aucun Lineo n'a
        // t dtect pour l'adversaire
        if(!find_lineo){
            new_liste.push_back(liste_IA[i]);
        }
    }
    // si la nouvelle liste n'est pas vide on remplace l'ancienne par la nouvelle
    if(new_liste.size() > 0){
        return new_liste;
    }
    // sinon on retourne la liste_IA tel quel car on ne peut rien faire dessus
    return liste_IA;
}

// calcul la meilleur liste de coup possible en mode difficile
vnoeud JoueurIA::listeDifficile(vnoeud liste_IA)
{
    vnoeud new_liste;
    Uint8 size_IA = liste_IA.size();

    // on parcour la liste et on vrifi si on a fait un Lineo
    // on garde tous les bons lments dans une liste  part
    for(Uint8 i=0;i<size_IA;i++){
        if(liste_IA[i].isLineo()){
            // on quitte dj en mettant un seul coup dans la liste car il est gagnant
            new_liste.push_back(liste_IA[i]);
            return new_liste;
        }
    }

    // on cr une liste des coup pour chaque coup que l'on fait et on vrifi que
    // a ne donne pas un Lineo  l'adversaire
    for(Uint8 i=0;i<size_IA;i++){
        // ce delay permet de ne pas monopoliser le processeur pour cecci
        SDL_Delay(IA_SDL_DELAY);

        vnoeud liste_adv = listeSousNoeuds(liste_IA[i]);
        Uint8 size_adv = liste_adv.size();
        bool find_no_lineo=true;
        bool find_lineo=false;
        for(Uint8 j=0;j<size_adv && find_no_lineo;j++){
            if(liste_adv[j].isLineo()){
                find_no_lineo = false;
            }else{
                vnoeud liste_ia = listeSousNoeuds(liste_adv[j]);
                Uint8 size_ia = liste_ia.size();
                for(Uint8 k=0;k<size_ia && !find_lineo;k++){
                    if(liste_ia[k].isLineo()){
                        find_lineo = true;
                    }
                }
            }
        }
        // on ajoute ce coup dans la liste temp si aucun Lineo n'a
        // t dtect pour l'adversaire
        if(find_no_lineo && find_lineo){
            new_liste.push_back(liste_IA[i]);
        }
    }
    // si la nouvelle liste n'est pas vide on remplace l'ancienne par la nouvelle
    if(new_liste.size() > 0){
        return new_liste;
    }
    // sinon on retourne la liste_IA tel quel car on ne peut rien faire dessus
    return liste_IA;
}

// calcul la meilleur liste de coup possible en mode impossible
vnoeud JoueurIA::listeImpossible(vnoeud liste_IA)
{
    vnoeud new_liste;
    Uint8 size_IA = liste_IA.size();

    // on parcour la liste et on vrifi si on a fait un Lineo
    // on garde tous les bons lments dans une liste  part
    for(Uint8 i=0;i<size_IA;i++){
        if(liste_IA[i].isLineo()){
            // on quitte dj en mettant un seul coup dans la liste car il est gagnant
            new_liste.push_back(liste_IA[i]);
            return new_liste;
        }
    }

    // on cr une liste des coup pour chaque coup que l'on fait et on vrifi que
    // a ne donne pas un Lineo  l'adversaire
    for(Uint8 i=0;i<size_IA;i++){
        // ce delay permet de ne pas monopoliser le processeur pour cecci
        SDL_Delay(IA_SDL_DELAY);

        vnoeud liste_adv = listeSousNoeuds(liste_IA[i]);
        Uint8 size_adv = liste_adv.size();
        bool find_no_lineo=true;
        bool find_lineo=false;
        for(Uint8 j=0;j<size_adv && find_no_lineo;j++){
            if(liste_adv[j].isLineo()){
                find_no_lineo = false;
            }else{
                vnoeud liste_ia = listeSousNoeuds(liste_adv[j]);
                Uint8 size_ia = liste_ia.size();
                for(Uint8 k=0;k<size_ia && !find_lineo;k++){
                    if(liste_ia[k].isLineo()){
                        find_lineo = true;
                    }
                }
            }
        }
        // on ajoute ce coup dans la liste temp si aucun Lineo n'a
        // t dtect pour l'adversaire
        if(find_no_lineo && find_lineo){
            new_liste.push_back(liste_IA[i]);
        }
    }
    // si la nouvelle liste n'est pas vide on remplace l'ancienne par la nouvelle
    if(new_liste.size() > 0){
        return new_liste;
    }
    // sinon on retourne la liste_IA tel quel car on ne peut rien faire dessus
    return liste_IA;
}

// on liste tous les coup que le joueur actuel va pouvoir faire
// on combine deux actions :
// 1) o poser la pice que l'on nous donne
// 2) qu'elle pice donner  l'adversaire
vnoeud JoueurIA::listeSousNoeuds(NoeudIA noeud)
{
    vnoeud liste;
    // on fait la liste des n des pices libres
    vector<Uint8> liste_pieces_libres;
    for(Uint8 i=0; i<16; i++){
        if(noeud.getPieceLibre(i)!=P_NO_FORME){
            liste_pieces_libres.push_back(i);
        }
    }
    Uint8 size = liste_pieces_libres.size();
    // ensuite on parcour toutes les places disponibles sur le plateau
    for(Uint8 i=0; i<4; i++){
        for(Uint8 j=0; j<4; j++){
            if(noeud.isCercleLibre(i,j)){
                for(Uint8 k=0; k<size; k++){
                    // on cr le nouveau sous noeud
                    NoeudIA ss_noeud = noeud;
                    // on modifier le noeud en jouant le coup
                    ss_noeud.placerPiece(i,j);
                    ss_noeud.choisirPiece(liste_pieces_libres[k]);
                    // on ajoute ce noeud  la liste des sous noeuds
                    liste.push_back(ss_noeud);
                }
            }
        }
    }

    return liste;
}

// toutes les pices sont sur le plateau sauf celle que l'on a dans les mains
// il ne reste donc qu' la poser au bon endroit
void JoueurIA::dernierCoup(NoeudIA noeud)
{
    // on parcour toutes les places disponibles sur le plateau
    //  la recherche de la dernire libre
    for(Uint8 i=0; i<4; i++){
        for(Uint8 j=0; j<4; j++){
            if(noeud.isCercleLibre(i,j)){
                // on l'a trouv alors on y pose la pice
                _IChoisi = i;
                _JChoisi = j;
                _PieceChoisie = P_NO_FORME;
            }
        }
    }
}


// Permet de dterminer un coup au hasard suivant les
// disponibilits de la donne actuelle du noeud en parmatres
void JoueurIA::noeudHasard(NoeudIA noeud)
{
    // on rcupre la liste des coups  jouer
    vnoeud liste = listeSousNoeuds(noeud);
    Uint16 max_rand = liste.size();
    // on initialise le gnrateur alatoire
    srand(SDL_GetTicks());
    // puis on choisit un nombre alatoire
    Uint16 i = rand() % max_rand;
    // on termine en choisissant le coup  faire
    _IChoisi = liste[i].getNextI();
    _JChoisi = liste[i].getNextJ();
    _PieceChoisie = liste[i].getNextPieceChoisie();
}

/*
// test une ligne afinde savoir ce qu'il y a dessus
void JoueurIA::testLigne(Uint8 piece_1, Uint8 piece_2, Uint8 piece_3, Uint8 piece_4, bool *duo, bool *trio)
{
    Uint8 piece_a=P_NO_FORME;
    Uint8 piece_b=P_NO_FORME;
    Uint8 piece_c=P_NO_FORME;

    // d'abord on compte le nombre de pices et on enregistre les trois premires
    Uint8 nb_piece=0;
    if(piece_1!=P_NO_FORME){
        nb_piece++;
        if(piece_a==P_NO_FORME){
            piece_a=piece_1;
        }else if(piece_b==P_NO_FORME){
            piece_b=piece_1;
        }else{
            piece_c=piece_1;
        }
    }
    if(piece_2!=P_NO_FORME){
        nb_piece++;
        if(piece_a==P_NO_FORME){
            piece_a=piece_2;
        }else if(piece_b==P_NO_FORME){
            piece_b=piece_2;
        }else{
            piece_c=piece_2;
        }
    }
    if(piece_3!=P_NO_FORME){
        nb_piece++;
        if(piece_a==P_NO_FORME){
            piece_a=piece_3;
        }else if(piece_b==P_NO_FORME){
            piece_b=piece_3;
        }else{
            piece_c=piece_3;
        }
    }
    if(piece_4!=P_NO_FORME){
        nb_piece++;
        if(piece_a==P_NO_FORME){
            piece_a=piece_4;
        }else if(piece_b==P_NO_FORME){
            piece_b=piece_4;
        }else{
            piece_c=piece_4;
        }
    }
    // on regarde maintenant si il y a un duo
    // dans ce cas seul piece_a et piece_b sont renseign
    // donc on ne teste qu'avec eux deux
    *duo = false;
    *trio = false;
    if(nb_piece==2){
        // on vrifie chacun des 8 critres
        if(piece_a & P_FORME_ROND && piece_b & P_FORME_ROND){
            *duo = true;
        }else if(piece_a & P_FORME_CARRE && piece_b & P_FORME_CARRE){
            *duo = true;
        }else if(piece_a & P_FORME_PETITE && piece_b & P_FORME_PETITE){
            *duo = true;
        }else if(piece_a & P_FORME_GRANDE && piece_b & P_FORME_GRANDE){
            *duo = true;
        }else if(piece_a & P_FORME_COULEUR_1 && piece_b & P_FORME_COULEUR_1){
            *duo = true;
        }else if(piece_a & P_FORME_VIOLETTE && piece_b & P_FORME_VIOLETTE){
            *duo = true;
        }else if(piece_a & P_FORME_PLEINE && piece_b & P_FORME_PLEINE){
            *duo = true;
        }else if(piece_a & P_FORME_CREUSE && piece_b & P_FORME_CREUSE){
            *duo = true;
        }
    // on regarde maintenant si il y a un trio
    // dans ce cas piece_a, piece_b et piece_c sont renseigns
    }else if(nb_piece==3){
        // on vrifie chacun des 8 critres
        if(piece_a & P_FORME_ROND && piece_b & P_FORME_ROND && piece_c & P_FORME_ROND){
            *trio = true;
        }else if(piece_a & P_FORME_CARRE && piece_b & P_FORME_CARRE && piece_c & P_FORME_CARRE){
            *trio = true;
        }else if(piece_a & P_FORME_PETITE && piece_b & P_FORME_PETITE && piece_c & P_FORME_PETITE){
            *trio = true;
        }else if(piece_a & P_FORME_GRANDE && piece_b & P_FORME_GRANDE && piece_c & P_FORME_GRANDE){
            *trio = true;
        }else if(piece_a & P_FORME_COULEUR_1 && piece_b & P_FORME_COULEUR_1 && piece_c & P_FORME_COULEUR_1){
            *trio = true;
        }else if(piece_a & P_FORME_VIOLETTE && piece_b & P_FORME_VIOLETTE && piece_c & P_FORME_VIOLETTE){
            *trio = true;
        }else if(piece_a & P_FORME_PLEINE && piece_b & P_FORME_PLEINE && piece_c & P_FORME_PLEINE){
            *trio = true;
        }else if(piece_a & P_FORME_CREUSE && piece_b & P_FORME_CREUSE && piece_c & P_FORME_CREUSE){
            *trio = true;
        }
    }
}
*/

