//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "JoueurNet.h"
#include "Joueur.h"
#include "Lineo.h"
#include "VueGame.h"

JoueurNet::JoueurNet(Lineo* app_lineo, Joueur* joueur)
{
	_AppLineo   = app_lineo;
	_Joueur     = joueur;
	_ComReseau  = new ComReseau(_AppLineo, EVENT_CREATE_VUE_CHOIXADVERSAIRE);
	_ThreadEtat = JNET_NO_ETAT;
	_QuiCommence=0;
	_ModeVueGameReseau = CHOIX_JOUER_MODE_RESEAU; // ncessaire car sinon on peut pas le passer en paramtre dans l'event
	_Nom="";
    // On dmarre le thread afin d'initialiser le joueur rseau et certaines donnes de la partie que le serveur nous donne
	_Thread     = NULL;
	_StopThread = false;
    executeThread(JNET_INIT);
}

JoueurNet::~JoueurNet()
{
    // on demande au thread de s'arrter
    _StopThread = true;
    // on attend que le thread se termine tout seul de faon  viter tout problme
    if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
    // on supprime toutes les classes instancies
    delete(_ComReseau);
}

// Cration d'un thread avec l'tat dsir
void JoueurNet::executeThread(Uint16 thread_etat)
{
    // on demande au thread actuel de se terminer au cas o il existerais encore
    _StopThread = true;
    // on attend que le prcdent thread soit termin
    if(_Thread != NULL) SDL_WaitThread(_Thread, NULL);
    // on retire le flag d'arrt du thread
    _StopThread = false;
    // on enregistre l'tat afin de pouvoir le rcuprer dans le thread
    _ThreadEtat = thread_etat;
    // on lance le thread en le crant, on enregistre son pointeur pour plus tard
    _Thread = SDL_CreateThread(this->staticThread, this);
    // on vrifit que le thread soit bien lanc, sinon on logue l'erreur
    if(_Thread == NULL) {
        fprintf(stderr, "\nImpossible de crer le thread : JoueurNet::executeThread(%d)\n%s\n\n",thread_etat,SDL_GetError());
    }
}

// Lance un thread dans la classe et excute une suite d'actions en fonction de l'tat transmis auparavant
int JoueurNet::thread(void *Param)
{
    Uint32 id_com=0;
    // On rcupre l'instance du parent
    JoueurNet *this_vue = (JoueurNet *)Param;
    // On switch sur l'tat prrenseign
    switch(this_vue->_ThreadEtat){
        // initialisation du joueur et de la partie en rcuprant des infos sur le serveur
        case JNET_INIT:{
            // On rcupre le nom du joueur rseau
            this_vue->_AppLineo->drawPopUp(POPUP_TYPE_INFO,this_vue->_AppLineo->_Langue->get("Partie en ligne"),this_vue->_AppLineo->_Langue->get("Dmarrage de la partie en cours..."),this_vue->_AppLineo->_Langue->get("Annuler"),EVENT_CREATE_VUE_CHOIXADVERSAIRE);
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            if(this_vue->_ComReseau->doAction(id_com,"NomAdversaire")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_Nom = this_vue->_ComReseau->get(id_com,"joueur");
            this_vue->_ComReseau->freeCom(id_com);
            this_vue->_Joueur->setNom(this_vue->_Nom);
            // on initalise le nom pour les phrases de la langue
            this_vue->_AppLineo->_Langue->replace("__NOM__",this_vue->_Nom);
            // On connait l'adversaire, on rcupre le nom du joueur qui commence et on le compare  l'adversaire
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            if(this_vue->_ComReseau->doAction(id_com,"QuiCommence")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            if(this_vue->_ComReseau->get(id_com,"joueur") == this_vue->_Nom){
                this_vue->_QuiCommence = JNET_JD_COMMENCE;
            }else{
                this_vue->_QuiCommence = JNET_JG_COMMENCE;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // Maintenant on signalle au serveur qu'on est prt et on attend que l'autre joueur le soit
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            bool first=true;
            do{
                // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                if(!first){
                    SDL_Delay(1000);
                }else{
                    first=false;
                }
                if(this_vue->_ComReseau->doAction(id_com,"JeSuisPret")!=CR_OK){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
            }while( this_vue->_ComReseau->get(id_com,"adversaire_pret")!="true"
                &&  this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"
                &&  !this_vue->_StopThread);
            // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
            if(this_vue->_StopThread){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // On a une rponse, on commence par supprimer la popup
            this_vue->_AppLineo->killPopUp();
            // Soit le joueur est toujours l et a dit qu'il est prt
            if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"){
                this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_START,&this_vue->_QuiCommence);
            // Soit il est plus l et donc on annule tout
            }else if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="false"){
                // on avertis que l'adversaire n'est plus disponible
                this_vue->_AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                                                this_vue->_AppLineo->_Langue->get("Partie en ligne"),
                                                this_vue->_AppLineo->_Langue->get("__NOM__ n'est plus disponible..."),
                                                this_vue->_AppLineo->_Langue->get("Retour  la liste des joueurs"),
                                                EVENT_CREATE_VUE_CHOIXADVERSAIRE);
            }
            this_vue->_ComReseau->freeCom(id_com);
        }break;
        case JNET_CERCLE_SELECTED:{
            // on initialise la communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"i", to_string(this_vue->_CercleSelected.i));
            this_vue->_ComReseau->set(id_com,"j", to_string(this_vue->_CercleSelected.j));
            this_vue->_ComReseau->set(id_com,"partie_ended", this_vue->_CercleSelected.partie_ended?"true":"false");
            if(this_vue->_CercleSelected.partie_ended){
                this_vue->_ComReseau->set(id_com,"partie", this_vue->_CercleSelected.partie);
            }
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur
            if(this_vue->_ComReseau->doAction(id_com,"CercleSelected")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // On s'assure que la partie est termine avant de le dire  VueGame
            if(this_vue->_ComReseau->get(id_com,"partie_ended")=="true"){
                this_vue->_ComReseau->freeCom(id_com);
                // on initialise une nouvelle communication
                id_com = this_vue->_ComReseau->initNewCom();
                this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
                bool first=true;
                do{
                    // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                    // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                    if(!first){
                        SDL_Delay(1000);
                    }else{
                        first=false;
                    }
                    if(this_vue->_ComReseau->doAction(id_com,"ConfirmPartieEndWait")!=CR_OK){
                        this_vue->_ComReseau->freeCom(id_com);
                        return -1;
                    }
                } while(this_vue->_ComReseau->get(id_com,"confirm_partie_end")!="true" && !this_vue->_StopThread);
                // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
                if(this_vue->_StopThread){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
                // on indique  VueGame le rsultat de la fin de partie
                if(this_vue->_ComReseau->get(id_com,"partie")=="win"){
                    this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_WIN);
                }else{
                    this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_NULLE);
                }
            }else{
                // maintenant on peut continuer la partie et on attend que le mme joueur choisisse une pice pour l'autre
                this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PIECE_WAIT);
            }
            this_vue->_ComReseau->freeCom(id_com);
        }break;
        case JNET_PIECE_SELECTED:
            // on initialise une nouvelle communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"autour_i", to_string((Uint16)this_vue->_PieceSelected));
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur
            if(this_vue->_ComReseau->doAction(id_com,"PieceSelected")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // et maintenant on peut demander  VueGame le changement de joueur
            this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_CHANGE_PLAYER);
        break;
        case JNET_CONFIRM_PARTIE_END:
            // on initialise une nouvelle communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur
            if(this_vue->_ComReseau->doAction(id_com,"ConfirmPartieEnd")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // on indique  VueGame le rsultat de la fin de partie
            if(this_vue->_ComReseau->get(id_com,"partie")=="lost"){
                this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_LOST);
            }else{
                this_vue->_AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_NULLE);
            }
            this_vue->_ComReseau->freeCom(id_com);
        break;
        // le but ici est juste de signaler au serveur qu'on existe toujours....
        case JNET_PING:
            // on initialise une nouvelle communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur jusqu' la demande d'arrter le thread par le flag
            while(!this_vue->_StopThread){
                if(this_vue->_ComReseau->doAction(id_com,"Ping")!=CR_OK){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
                // on fait une pause de 30 secondes  chaque fois afin ne pas charger le serveur et le processeur local
                // mais plutt que de faire un SDL_Delay(30000); qui empecherait le thread de se terminer malgr le flag
                // on fait une novuelle boucle avec le flag et un court SDL_Delay
                Uint16 cpt = 0;
                while(cpt<300 && !this_vue->_StopThread){
                    SDL_Delay(100);
                    cpt++;
                }
            }
            this_vue->_ComReseau->freeCom(id_com);
        break;
        // demande d'une nouvelle partie au serveur ( l'adversaire)
        case JNET_NEW_PARTIE:{
            // on initialise une nouvelle communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on fait la demande au serveur
            if(this_vue->_ComReseau->doAction(id_com,"NewPartie")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // on vrifie que l'adversaire soit toujours prsent
            if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"){
                this_vue->_ComReseau->freeCom(id_com);
                // on avertis qu'on attend la rponse de l'adversaire
                this_vue->_AppLineo->drawPopUp( POPUP_TYPE_INFO,
                                                this_vue->_AppLineo->_Langue->get("Partie en ligne"),
                                                this_vue->_AppLineo->_Langue->get("Demande d'une nouvelle partie avec __NOM__ en cours..."),
                                                this_vue->_AppLineo->_Langue->get("Annuler"),
                                                EVENT_GAME_JE_VEUX_PLUS_NEW_PARTIE);
                // on attend la rponse de l'adversaire tant qu'on appuie pas sur Annuler
                id_com = this_vue->_ComReseau->initNewCom();
                this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
                bool first=true;
                do{
                    // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                    // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                    if(!first){
                        SDL_Delay(1000);
                    }else{
                        first=false;
                    }
                    if(this_vue->_ComReseau->doAction(id_com,"ReponseNewPartieWait")!=CR_OK){
                        this_vue->_ComReseau->freeCom(id_com);
                        return -1;
                    }
                }while(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"
                    && this_vue->_ComReseau->get(id_com,"reponse_donnee")=="false"
                    && !this_vue->_StopThread);
                // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
                if(this_vue->_StopThread){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
                // on ferme la popup afin d'tre sr de pouvoir envoyer des events
                this_vue->_AppLineo->killPopUp();
                // l'autre a rpondu ou il n'est plus l, on agit en consquence
                if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"
                && this_vue->_ComReseau->get(id_com,"reponse_donnee")=="true"
                && this_vue->_ComReseau->get(id_com,"reponse")=="oui"){
                    // on initialise maintenant la nouvelle partie car tout est OK
                    this_vue->_AppLineo->_Events->addEvent(EVENT_CREATE_VUE_GAME,&this_vue->_ModeVueGameReseau);
                }else if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="false"){
                    // l'autre joueur n'tant plus prsent avec le user on lui indique
                    this_vue->_AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                                                    this_vue->_AppLineo->_Langue->get("Partie en ligne"),
                                                    this_vue->_AppLineo->_Langue->get("__NOM__ n'est plus disponible..."),
                                                    this_vue->_AppLineo->_Langue->get("Retour  la liste des joueurs"),
                                                    EVENT_CREATE_VUE_CHOIXADVERSAIRE);
                }else{
                    // l'autre joueur ne veut pas jouer avec le user :-S alors on lui dit.... douche froide ! :-P
                    this_vue->_AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                                                    _AppLineo->_Langue->get("Partie en ligne"),
                                                    this_vue->_AppLineo->_Langue->get("__NOM__ ne veut pas jouer avec vous..."),
                                                    _AppLineo->_Langue->get("Retour  la liste des joueurs"),
                                                    EVENT_CREATE_VUE_CHOIXADVERSAIRE);
                }
            }else{
                // on avertis que l'adversaire n'est plus disponible
                this_vue->_AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                                                this_vue->_AppLineo->_Langue->get("Partie en ligne"),
                                                this_vue->_AppLineo->_Langue->get("__NOM__ n'est plus disponible..."),
                                                this_vue->_AppLineo->_Langue->get("Retour  la liste des joueurs"),
                                                EVENT_CREATE_VUE_CHOIXADVERSAIRE);
            }
            this_vue->_ComReseau->freeCom(id_com);
        }break;
        // informe le serveur qu'on ne veut plus faire de nouvelle partie
        case JNET_VEUX_PLUS_NEW_PARTIE:
            // on initialise une nouvelle communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on fait la demande au serveur
            if(this_vue->_ComReseau->doAction(id_com,"JeVeuxPlusNewPartie")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // on retourne  la liste des joueurs
            this_vue->_AppLineo->_Events->addEvent(EVENT_CREATE_VUE_CHOIXADVERSAIRE);
        break;
        // attend une demande de nouvelle partie de l'adversaire
        case JNET_NEW_PARTIE_WAIT:{
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            bool first=true;
            do{
                // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                if(!first){
                    SDL_Delay(1000);
                }else{
                    first=false;
                }
                if(this_vue->_ComReseau->doAction(id_com,"NewPartieWait")!=CR_OK){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
            }while(this_vue->_ComReseau->get(id_com,"demande")=="false" && !this_vue->_StopThread);
            this_vue->_ComReseau->freeCom(id_com);
            // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
            if(this_vue->_StopThread){
                return -1;
            }
            // si on est l c'est qu'il y a eu une demande
            // on en informe alors le user via une popup
            this_vue->_AppLineo->drawPopUp( POPUP_TYPE_INFO,
                                            _AppLineo->_Langue->get("Partie en ligne"),
                                            this_vue->_AppLineo->_Langue->get("__NOM__ veut faire une nouvelle partie avec vous !"),
                                            _AppLineo->_Langue->get("Accepter"),
                                            EVENT_GAME_REPONSE_NEW_PARTIE_OUI,
                                            _AppLineo->_Langue->get("Refuser"),
                                            EVENT_GAME_REPONSE_NEW_PARTIE_NON);
        }break;
        // donne une rponse positive au serveur pour une nouvelle partie
        case JNET_REPONSE_NEW_PARTIE_OUI:
            // on initialise la communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"reponse", "oui");
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur
            if(this_vue->_ComReseau->doAction(id_com,"ReponseNewPartie")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // On s'assure que l'adversaire attend toujours
            if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"){
                // on initialise maintenant la nouvelle partie car tout est OK
                this_vue->_AppLineo->_Events->addEvent(EVENT_CREATE_VUE_GAME,&this_vue->_ModeVueGameReseau);
            }else{
                // l'autre joueur n'tant plus prsent avec le user on lui indique
                this_vue->_AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                                                this_vue->_AppLineo->_Langue->get("Partie en ligne"),
                                                this_vue->_AppLineo->_Langue->get("__NOM__ n'est plus disponible..."),
                                                this_vue->_AppLineo->_Langue->get("Retour  la liste des joueurs"),
                                                EVENT_CREATE_VUE_CHOIXADVERSAIRE);
            }
            this_vue->_ComReseau->freeCom(id_com);
        break;
        // donne une rponse ngative au serveur pour une nouvelle partie
        case JNET_REPONSE_NEW_PARTIE_NON:
            // on initialise la communication
            id_com = this_vue->_ComReseau->initNewCom();
            // on paramtre toutes les options qu'on va envoyer
            this_vue->_ComReseau->set(id_com,"reponse", "non");
            this_vue->_ComReseau->set(id_com,"session",this_vue->_AppLineo->_SessionReseau);
            // on envoi l'action au serveur
            if(this_vue->_ComReseau->doAction(id_com,"ReponseNewPartie")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // on retourne  la liste des joueurs
            this_vue->_AppLineo->_Events->addEvent(EVENT_CREATE_VUE_CHOIXADVERSAIRE);
        break;
    }
    // on rinitialise l'tat du thread afin d'tre sr qu'il ne fasse rien tout seul la prochaine fois
    // c'est plus pour viter des erreurs de codage (tourderies) qu'autre chose... ;)
    this_vue->_ThreadEtat = JNET_NO_ETAT;
    // c'est fini on quitte le thread
    return 0;
}

// on attend et rcupre la pice choisie par le joueur via le rseau
void JoueurNet::pieceWait(VueGame *vue_game)
{
    Uint32 id_com = _ComReseau->initNewCom();
    _ComReseau->set(id_com,"session",_AppLineo->_SessionReseau);
    bool first=true;
    do{
        // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
        // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
        if(!first){
            SDL_Delay(1000); // attente d'une seconde pour ne pas surcharger le serveur
        }else{
            first=false;
        }
        if(_ComReseau->doAction(id_com,"PieceWait")!=CR_OK){
            _ComReseau->freeCom(id_com);
            return;
        }
    }while(_ComReseau->get(id_com,"piece_selected")!="true"
        && _ComReseau->get(id_com,"actif")=="true"
        && !vue_game->getStopThread());
    // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
    if(vue_game->getStopThread()){
        _ComReseau->freeCom(id_com);
        return;
    }
    // on s'assure que l'adversaire soit toujours actif
    if(_ComReseau->get(id_com,"actif") == "true"){
        // la pice est choisie alors on la rcupre et on l'envois a VueGame
        _PieceChoisie = string_to_int( _ComReseau->get(id_com,"autour_i"));
        _AppLineo->_Events->addEvent(EVENT_GAME_PIECE_SELECTED,&_PieceChoisie);
    }else{
        // l'adversaire n'est plus actif alors on en informe le user afin de stopper la partie
        _AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                            _AppLineo->_Langue->get("Partie en ligne"),
                            _AppLineo->_Langue->get("__NOM__ n'est plus disponible...La partie est termine."),
                            _AppLineo->_Langue->get("Retour  la liste des joueurs"),EVENT_CREATE_VUE_CHOIXADVERSAIRE);
    }
    _ComReseau->freeCom(id_com);
}

// on attend et rcupre le cercle choisi par le joueur via le rseau
void JoueurNet::cercleWait(VueGame *vue_game)
{
    Uint32 id_com = _ComReseau->initNewCom();
    _ComReseau->set(id_com,"session",_AppLineo->_SessionReseau);
    bool first=true;
    do{
        // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
        // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
        if(!first){
            SDL_Delay(1000); // attente d'une seconde pour ne pas surcharger le serveur
        }else{
            first=false;
        }
        if(_ComReseau->doAction(id_com,"CercleWait")!=CR_OK){
            _ComReseau->freeCom(id_com);
            return;
        }
    }while(_ComReseau->get(id_com,"cercle_selected")!="true"
        && _ComReseau->get(id_com,"actif")=="true"
        && !vue_game->getStopThread());
    // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
    if(vue_game->getStopThread()){
        _ComReseau->freeCom(id_com);
        return;
    }
    // on s'assure que l'adversaire soit toujours actif
    if(_ComReseau->get(id_com,"actif") == "true"){
        // le cercle est choisi alors on le rcupre et on l'envois a VueGame
        _IChoisi = string_to_int( _ComReseau->get(id_com,"i"));
        _JChoisi = string_to_int( _ComReseau->get(id_com,"j"));
        _AppLineo->_Events->addEvent(EVENT_GAME_CERCLE_SELECTED,&_IChoisi,&_JChoisi);
    }else{
        // l'adversaire n'est plus actif alors on en informe le user afin de stopper la partie
        _AppLineo->drawPopUp( POPUP_TYPE_WARNING,
                            _AppLineo->_Langue->get("Partie en ligne"),
                            _AppLineo->_Langue->get("__NOM__ n'est plus disponible...La partie est termine."),
                            _AppLineo->_Langue->get("Retour  la liste des joueurs"),EVENT_CREATE_VUE_CHOIXADVERSAIRE);
    }
    _ComReseau->freeCom(id_com);
}

void JoueurNet::confirmePartieEnd()
{
    // on lance le thread afin de confirmer la fin de partie au serveur
	executeThread(JNET_CONFIRM_PARTIE_END);
}

// on informe le serveur de la pice choisie
void JoueurNet::pieceSelected(Uint8 autour_i)
{
    // on enregistre la pice afin de pouvoir rcuprer la valeur dans le thread
    _PieceSelected = (Uint16)autour_i;
    // puis on lance le thread afin de librer la main de Lino client et donc  l'humain
	executeThread(JNET_PIECE_SELECTED);
}

// on informe le serveur de l'emplacement sur le plateau pour la pice choisie
void JoueurNet::cercleSelected(Uint8 i, Uint8 j, bool partie_ended, string partie)
{
    // on remplit une structure de la classe afin de pouvoir rcuprer les valeurs dans le thread
    _CercleSelected.i               = (Uint16)i;
    _CercleSelected.j               = (Uint16)j;
    _CercleSelected.partie_ended    = partie_ended;
    _CercleSelected.partie          = partie;
    // puis on lance le thread afin de librer la main de Lino client et donc  l'humain
	executeThread(JNET_CERCLE_SELECTED);
}

// on attend en interrogeant le serveur que l'adversaire nous demande de refaire une partie avec lui
void JoueurNet::newPartieWait()
{
    // on lance le thread afin de faire la demande en boucle
	executeThread(JNET_NEW_PARTIE_WAIT);
}

// gre les vnements de la vue
void JoueurNet::processEvents(SDL_Event *event)
{
    if (event->type == SDL_USEREVENT){
        switch (event->user.code){
            case EVENT_GAME_PING:
                // on lance la boucle du ping
                executeThread(JNET_PING);
            break;
            case EVENT_GAME_NEW_PARTIE:
                // on lance le thread afin de faire la demande au serveur
                executeThread(JNET_NEW_PARTIE);
            break;
            case EVENT_GAME_JE_VEUX_PLUS_NEW_PARTIE:
                // on lance le thread pour prvenir le serveur
                executeThread(JNET_VEUX_PLUS_NEW_PARTIE);
            break;
            case EVENT_GAME_REPONSE_NEW_PARTIE_OUI:
                // on lance le thread pour dire OUI
                executeThread(JNET_REPONSE_NEW_PARTIE_OUI);
            break;
            case EVENT_GAME_REPONSE_NEW_PARTIE_NON:
                // on lance le thread pour dire NON
                executeThread(JNET_REPONSE_NEW_PARTIE_NON);
            break;
        }
    }
}

