//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __JOUEURNET_H__
#define __JOUEURNET_H__

#include <SDL.h>
#include <SDL_thread.h>
#include <string>

#include "LaDonne.h"
#include "ComReseau.h"
#include "Events.h"
#include "Templates.h"


enum enum_Threads_Etats
{
    JNET_NO_ETAT,
    JNET_INIT,
    JNET_CERCLE_SELECTED,
    JNET_PIECE_SELECTED,
    JNET_CONFIRM_PARTIE_END,
    JNET_PING,
    JNET_NEW_PARTIE,
    JNET_VEUX_PLUS_NEW_PARTIE,
    JNET_NEW_PARTIE_WAIT,
    JNET_REPONSE_NEW_PARTIE_OUI,
    JNET_REPONSE_NEW_PARTIE_NON
};

#define JNET_JG_COMMENCE 0
#define JNET_JD_COMMENCE 1

class Lineo;
class Joueur;
class VueGame;

struct CercleSelected {
    Uint16 i;
    Uint16 j;
    bool partie_ended;
    string partie;
};

class JoueurNet
{
  //*******************
  //* variables membres
	private:
        Lineo          *_AppLineo;
        Joueur         *_Joueur;

        Uint8          _PieceChoisie;
        Uint8          _IChoisi;
        Uint8          _JChoisi;
        Uint8          _QuiCommence;

        CercleSelected  _CercleSelected;
        Uint16          _PieceSelected;

        string         _Nom;

        Uint8          _ThreadEtat;
        SDL_Thread 	   *_Thread;
        bool           _StopThread;

        ComReseau      *_ComReseau;
        Uint8          _ModeVueGameReseau;
        Uint32         _TxtPlayerDId;
        bool           _QuitPartie;

  //*******************
  //* fonctions membres
	public:
		JoueurNet(Lineo* app_lineo,Joueur* joueur);
		~JoueurNet();
		void pieceWait(VueGame *vue_game);
		void cercleWait(VueGame *vue_game);
		void confirmePartieEnd();
		void newPartieWait();
		void processEvents(SDL_Event *event);
        void pieceSelected(Uint8 autour_i);
        void cercleSelected(Uint8 i, Uint8 j, bool partie_ended, string partie);

        inline Uint8 getQuiCommence(){
			return _QuiCommence;
		}

    private:
        void executeThread(Uint16 thread_etat);
        static int staticThread( void *Param ){
            JoueurNet* Obj = reinterpret_cast<JoueurNet*>( Param );
            return Obj->thread(Param);
        }
        int thread(void *Param);
};

#endif // __JOUEURNET_H__
