//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Lineo.h"

Lineo::Lineo(int argc, char * argv[])
{
	// on initialise Lineo
	init(argc, argv);
    // on lance la boucle principale
    run();
}

Lineo::~Lineo()
{
    delete(_Screen);
    delete(_VFont);
    delete(_VTexte);
    delete(_Curseur);
    delete(_Events);
    delete(_Config);
    delete(_Langue);
    delete(_PopUp);
    Musique::kill();
}

void Lineo::init(int argc, char * argv[])
{
	// on initialise les variables membres
	initVariables();
    // on traite les arguments s'il y en a
    parseArgs(argc, argv);
  // on initialise la fentre du soft
    videoSetup();
    // on initialise la classe Screen
#ifdef Nokia770
    _Screen->setVideoMode(true);
#else
    if(_Config->get("mode_video")=="plein_ecran"){
        if(!_Screen->setVideoMode(true)){
            // le plein cran n'a pas march alors on remet en fentr
            _Config->set("mode_video","fenetre");
        }
    }
    if(_Config->get("mode_video")=="fenetre"){
        _Screen->setVideoMode(false);
    }
#endif
    // on initialise la rptition des touches et on prend en charg l'unicode
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
    SDL_EnableUNICODE(1);
    // on initialise la musique du jeu
    if(_Config->get("musique")=="ON"){
        // on initialise la musique
        Musique::getInstance()->init();
    }
    // on cr les fonts
    _VFont->add(FONT_12_WHITE,(string)(DATA_PATH)+"fonts/arial_12_white_blueout_flat","png");
    _VFont->add(FONT_18_WHITE,(string)(DATA_PATH)+"fonts/arial_18_white_blackout_flat","png");
    _VFont->add(FONT_24_WHITE,(string)(DATA_PATH)+"fonts/arial_24_white_blackout_flat","png");
    _VFont->add(FONT_26_WHITE,(string)(DATA_PATH)+"fonts/arial_26_white_blueout_flat","png");
    _VFont->add(FONT_26_YELLOW,(string)(DATA_PATH)+"fonts/arial_26_yellow_blueout_flat","png");
    _VFont->add(FONT_36_WHITE,(string)(DATA_PATH)+"fonts/arial_36_white_blueout_flat","png");
    _VFont->add(FONT_36_YELLOW,(string)(DATA_PATH)+"fonts/arial_36_yellow_blueout_flat","png");
    _VFont->add(FONT_58_WHITE,(string)(DATA_PATH)+"fonts/arial_58_white_blueout_flat","png");
    // on initialise la classe Curseur
    _Curseur->init();
#ifndef Nokia770
    // on lance le thread permettant de mettre  jour l'url
    _Thread = SDL_CreateThread(this->staticUpdateUrlServeurThread, this);
    // on vrifit que le thread soit bien lanc, sinon on logue l'erreur
    if(_Thread == NULL) {
        fprintf(stderr, "\nImpossible de crer le thread : Lineo::staticUpdateUrlServeurThread()\n%s\n\n",SDL_GetError());
    }
#endif
    // on lance le message pour afficher la premire vue
    _Events->addEvent(EVENT_CREATE_VUE_GENERIQUE_DEBUT);
}

// initialise plusieurs variables de la classe et du logiciel
void Lineo::initVariables(void)
{
    // on initialise les variables membres
    _ShowFps = false;
    _DebugMode = false;
    // on initialise les classes qui seront appele grce  l'application Lino
    _Config 	= new Config();
    _Curseur 	= new Curseur();
    _Events 	= new Events();
    _Screen 	= new Screen(this);
    _VFont 		= new VFont(this);
    _VTexte 	= new VTexte(this);
    _Langue 	= new Langue(this);
    // Seed random number generator
    srand(SDL_GetTicks());
    // on initialise toutes les vues
    _VueActive=VUE_NO_VUE;
    _GeneriqueDebut=NULL;
    _VueMainMenu=NULL;
    _VueGame=NULL;
    _VueNiveau=NULL;
    _VueOption=NULL;
    _VueConstruction=NULL;
    _VueSousOption=NULL;
    _VueChoixCouleurPiece=NULL;
    _VueChoixLangue=NULL;
    _VueChoixJouer=NULL;
    _VueCredits=NULL;
    _GeneriqueFin=NULL;
#ifndef Nokia770
    _VueChoixModeVideo=NULL;
    _VueChoixAdversaire=NULL;
    _VueLogin=NULL;
    // on initialise le pointeur du thread
    _Thread=NULL;
    // on initialise la variable de session pour le rseau
    _SessionReseau="NULL";
#endif
    // on initialise la popup
    _PopUp=NULL;
}

// initialise la fentre du soft
void Lineo::videoSetup(void)
{
    // Init SDL Video:
    if (SDL_Init(SDL_INIT_VIDEO| SDL_INIT_AUDIO)< 0){
        fprintf(stderr,
              "\nErreur: Je ne peux pas initialiser la vido!\n"
              "L'erreur que SDL a renvoy tait:\n"
              "%s\n\n", SDL_GetError());
        exit(1);
    }

    // Set window manager stuff
    SDL_WM_SetCaption("Lineo " VERSION, "Lineo");

    // met en place l'icne du Soft
    setIcon();
}

// Met en place l'icne du Soft
void Lineo::setIcon(void)
{
    SDL_Surface * icon;

    // Load icon into a surface
    icon = IMG_Load(((string)(DATA_PATH)+"icon").c_str());
    if (icon == NULL) {
        fprintf(stderr,
              "\nErreur: Je ne peux pas charger l'image de l'icne: %s\n"
              "L'erreur que SDL a renvoy tait:\n"
              "%s\n\n", ((string)(DATA_PATH)+"icon").c_str(), SDL_GetError());
        exit(1);
    }

    // Set icon
    SDL_WM_SetIcon(icon, NULL);

    // Free icon surface
    SDL_FreeSurface(icon);
}

// Parse command-line arguments
void Lineo::parseArgs(int argc, char * argv[])
{
    int i;

    // Parse arguments
    for (i = 1; i < argc; i++) {
        if (strcmp(argv[i], "--show-fps") == 0) {
            _ShowFps = true;
        } else if (strcmp(argv[i], "--debug-mode") == 0) {
            _DebugMode = true;
        } else if (strcmp(argv[i], "--help") == 0) {
            puts("Lino " VERSION "\n"
                 "  Merci de lire le fichier \"README.txt\" pour plus de dtails.\n");
            exit(0);
        } else {
            // affiche le message d'usage
            printf("Usage: %s [--help] [--show-fps] [--debug-mode]\n", argv[0]);
            exit(0);
        }
    }
}

// boucle principale du jeu avec la gestion des vnements
void Lineo::run(void)
{
    bool exit=false;
    // on lance le timer qui sera utile  plein de monde
    Timer *timer = new Timer(this);
    // on dmarre la musique
    Musique::getInstance()->play();
    // on lance la grande boucle qui bouclera jusqu' la fin du soft
    while(!exit){
		exit=processEvents();

        // on raffraichi la scne si elle en a besoin (nottament pour les animations)
        switch(_VueActive){
            case VUE_CREDITS:
                _VueCredits->refreshScreen();
            break;
        }
		// on rafraichit l'animation de la PopUp si il y en a une d'affiche
		if(_PopUp!=NULL){
			_PopUp->refreshScreen();
		}

        //on dessine la scne
        _Screen->draw();
        SDL_Delay(10);
    }
    // on arrte le timer qui est n'est plus utile
    delete(timer);
    // on dit aurevoir ;)
    printf("@+\n");
    // Quitte SDL et ferme donc le programme...
    SDL_Quit();
}

#ifndef Nokia770
    // met  jour de l'url du serveur dans un thread
    int Lineo::updateUrlServeurThread(void *Param)
    {
        // On rcupre l'instance du parent
        Lineo *this_lineo = (Lineo *)Param;
        ComReseau *com_reseau  = new ComReseau(this_lineo,EVENT_0);
        Uint32 id_com = com_reseau->initNewCom(false);
        if(com_reseau->doAction(id_com,"GetUrlServeur")!=CR_OK) return -1;
        this_lineo->_Config->set("serveur",com_reseau->get(id_com,"serveur"));
        com_reseau->freeCom(id_com);
        return 0;
    }
#endif

bool Lineo::processEvents(void)
{
    SDL_Event event;
    while(SDL_PollEvent(&event)){
		// on vrifie qu'aucune PopUp ne soit affiche actuellement
		// car si c'est le cas aucun vnement ne doit tre gr tant
		// que la popup n'est pas ferme
		if(_PopUp==NULL){
			// on gre d'abord les diffrents vnements du logiciel
	        switch (event.type) {
	        	case SDL_USEREVENT:
	                switch (event.user.code) {
	                    case EVENT_CREATE_VUE_GENERIQUE_DEBUT:
	                        // On libre l'cran
	                        freeVues();
	                        _GeneriqueDebut = new GeneriqueDebut(this);
	                        _VueActive=VUE_GENERIQUE_DEBUT;
	                    break;
	                    case EVENT_CREATE_VUE_GENERIQUE_FIN:
	                        // On libre l'cran
	                        freeVues();
	                        _GeneriqueFin = new GeneriqueFin(this);
	                        _VueActive=VUE_GENERIQUE_FIN;
	                    break;
	                    case EVENT_CREATE_VUE_MAIN_MENU:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Main Menu
	                        _VueMainMenu = new VueMainMenu(this);
	                        _VueActive=VUE_MAIN_MENU;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIX_JOUER:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Choix Jouer
	                        _VueChoixJouer = new VueChoixJouer(this);
	                        _VueActive=VUE_CHOIX_JOUER;
	                    break;
	                    case EVENT_CREATE_VUE_GAME:{
	                        // on rcupre le mode de jeux avant de librer la vue
	                        Uint16 mode_jeu = *(Uint16*)event.user.data1;
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Game
	                        _VueGame = new VueGame(this, mode_jeu);
	                        _VueActive=VUE_GAME;
	                    }break;
	                    case EVENT_CREATE_VUE_NIVEAU:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Niveau
	                        _VueNiveau = new VueNiveau(this);
	                        _VueActive=VUE_NIVEAU;
	                    break;
	                    case EVENT_CREATE_VUE_OPTION:{
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue option
	                        _VueOption = new VueOption(this, EVENT_CREATE_VUE_MAIN_MENU);
	                        _VueActive=VUE_OPTION;
	                    }break;
	                    case EVENT_OPTION_JOUEUR:
	                    case EVENT_OPTION_MUSIQUE:
	                    case EVENT_OPTION_VUE_DESSUS:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cr la vue VueSousOption
	                        _VueSousOption = new VueSousOption(this, event.user.code);
	                        _VueActive=VUE_SOUSOPTION;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIX_COULEUR_PIECE:{
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue du choix de la couleur des pices
	                        _VueChoixCouleurPiece = new VueChoixCouleurPiece(this);
	                        _VueActive=VUE_CHOIX_COULEUR_PIECE;
	                    }break;
	                    case EVENT_CREATE_VUE_CHOIX_LANGUE:{
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue du choix de la langue
	                        _VueChoixLangue = new VueChoixLangue(this);
	                        _VueActive=VUE_CHOIX_LANGUE;
	                    }break;
	                    case EVENT_CREATE_VUE_CONSTRUCTION:{
	                        // on rcupre la provenance
	                        Uint16 evnmt_provenance = *(Uint16*)event.user.data1;
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Joueur
	                        _VueConstruction = new VueConstruction(this, evnmt_provenance);
	                        _VueActive=VUE_CONSTRUCTION;
	                    }break;
	                    case EVENT_CREATE_VUE_CREDITS:{
	                        // on rcupre l'evenement de retour
	                        Uint16 event_retour = *(Uint16*)event.user.data1;
	                        // On libre l'cran
	                        freeVues();
	                        // on cre les crdits
	                        _VueCredits = new VueCredits(this, event_retour);
	                        _VueActive=VUE_CREDITS;
	                    }break;
#ifndef Nokia770 // ------------------------------------------------------------
                        case EVENT_CREATE_VUE_LOGIN:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Choix Jouer
	                        _VueLogin = new VueLogin(this);
	                        _VueActive=VUE_LOGIN;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIXADVERSAIRE:
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue Choix Jouer
	                        _VueChoixAdversaire = new VueChoixAdversaire(this);
	                        _VueActive=VUE_CHOIXADVERSAIRE;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIX_MODE_VIDEO:{
	                        // on libre les vues ventuellements affiches
	                        freeVues();
	                        // puis on cre la vue du choix de mode vido
	                        _VueChoixModeVideo = new VueChoixModeVideo(this);
	                        _VueActive=VUE_CHOIX_MODE_VIDEO;
	                    }break;
	                    case EVENT_CREATE_VUE_HOW_TO_PLAY:{
							#ifdef __unix
								string fichier=DATA_PATH + "aide/comment_jouer_"+_Langue->get("iso")+".html &";

								string commande="/usr/bin/sensible-browser "; //fonctionne pour les systmes Debian
								if (system((commande+fichier).c_str())== 0) break;

								commande="/usr/bin/firefox "; //devrait fonctionner pour plein d'autres.
								if (system((commande+fichier).c_str())== 0) break;
								printf("default browser not found\n"); //il faut trouver autre chose
							#endif
							#ifdef __WIN32
		                        string fichier=".\\datas\\aide\\comment_jouer_"+_Langue->get("iso")+".html";
		                        ShellExecute(NULL,  // Handle de la fentre parent
		                         "open", // Action  effectuer
		                         fichier.c_str(), // Fichier
		                         NULL, // Paramtres
		                         NULL, // Rpertoire par dfaut
		                         SW_SHOWNORMAL // Manire d'afficher
		                         );
	                    	#endif
	                    }break;
#endif // ----------------------------------------------------------------------
	                    case EVENT_QUITTER:
	                        return true;
	                    break;
	                }
	            break;
	        	default:
	    	    break;
	    	}
	        // puis les vnements des vues
	        switch(_VueActive){
	            case VUE_GENERIQUE_DEBUT:
	                _GeneriqueDebut->processEvents(&event);
	            break;
	            case VUE_GENERIQUE_FIN:
	                _GeneriqueFin->processEvents(&event);
	            break;
	            case VUE_MAIN_MENU:
	                _VueMainMenu->processEvents(&event);
	            break;
	            case VUE_CHOIX_JOUER:
	                _VueChoixJouer->processEvents(&event);
	            break;
	            case VUE_GAME:
	                _VueGame->processEvents(&event);
	            break;
	            case VUE_NIVEAU:
	                _VueNiveau->processEvents(&event);
	            break;
	            case VUE_OPTION:
	                _VueOption->processEvents(&event);
	            break;
	            case VUE_SOUSOPTION:
	                _VueSousOption->processEvents(&event);
	            break;
	            case VUE_CHOIX_COULEUR_PIECE:
	                _VueChoixCouleurPiece->processEvents(&event);
	            break;
	            case VUE_CHOIX_LANGUE:
	                _VueChoixLangue->processEvents(&event);
	            break;
	            case VUE_CONSTRUCTION:
	                _VueConstruction->processEvents(&event);
	            break;
	            case VUE_CREDITS:
	                _VueCredits->processEvents(&event);
	            break;
#ifndef Nokia770 // ------------------------------------------------------------
	            case VUE_LOGIN:
	                _VueLogin->processEvents(&event);
	            break;
	            case VUE_CHOIXADVERSAIRE:
	                _VueChoixAdversaire->processEvents(&event);
	            break;
				case VUE_CHOIX_MODE_VIDEO:
	                _VueChoixModeVideo->processEvents(&event);
	            break;
#endif // ----------------------------------------------------------------------
	        }
		}else{
			// une PopUp est actuellement affiche alors on en traite *QUE SES* vnements
			_PopUp->processEvents(&event);
			// on ferme la popup si elle nous l'a demand, c'est le seul vnement
			// extrieur qui peut tre trait ici quand une popup est affiche
	        if(event.type==SDL_USEREVENT && event.user.code==EVENT_CLOSE_POPUP){
				// il peut y avoir un event  appeler dans data1
				if(event.user.data1 != 0){
			    	Uint32 new_event = *(Uint32*)event.user.data1;
					_Events->addEvent(new_event);
				}
				killPopUp();
			}
		}
		// pour quitter on le fait quand le user dcide, donc peu importe
		// qu'il y ai une PopUp ou pas
        if(event.type==SDL_QUIT){
			// on vrifit tout de mme qu'il n'y a pas une PopUp
			// si c'est le cas on la ferme
			killPopUp();
            // plutt que de quitter brutallement on fait comme si on avait
            // appuy sur le bouton quitter du menu principal
            static Uint16 event_retour=EVENT_CREATE_VUE_GENERIQUE_FIN;
            _Events->addEvent(EVENT_CREATE_VUE_CREDITS,&event_retour);
		}
    }

    return false;
}

// libre toutes les vues
void Lineo::freeVues(void)
{
    delete(_GeneriqueDebut);
    delete(_GeneriqueFin);
    delete(_VueMainMenu);
    delete(_VueGame);
    delete(_VueNiveau);
    delete(_VueOption);
    delete(_VueConstruction);
    delete(_VueSousOption);
    delete(_VueChoixCouleurPiece);
    delete(_VueChoixJouer);
    delete(_VueChoixLangue);
    delete(_VueCredits);
#ifndef Nokia770
    delete(_VueLogin);
    delete(_VueChoixAdversaire);
	delete(_VueChoixModeVideo);
#endif

    _GeneriqueDebut=NULL;
    _GeneriqueFin=NULL;
    _VueMainMenu=NULL;
    _VueGame=NULL;
    _VueNiveau=NULL;
    _VueOption=NULL;
    _VueConstruction=NULL;
    _VueSousOption=NULL;
    _VueChoixCouleurPiece=NULL;
    _VueChoixJouer=NULL;
    _VueChoixLangue=NULL;
    _VueCredits=NULL;
#ifndef Nokia770
    _VueLogin=NULL;
    _VueChoixModeVideo=NULL;
    _VueChoixAdversaire=NULL;
#endif
}

// toutes ces fonctions affichent une popup  l'cran avec 0  3 boutons
void Lineo::drawPopUp(Uint8 type, string titre, string texte)
{
	drawPopUp(type, titre, texte, "", 0, "", 0, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt)
{
	drawPopUp(type, titre, texte, btn1_txt, btn1_evt, "", 0, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt)
{
	drawPopUp(type, titre, texte, btn1_txt, btn1_evt, btn2_txt, btn2_evt, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt, string btn3_txt, Uint32 btn3_evt)
{
	// si une popup tait dj affiche on la supprime (comme une erreur rseau alors qu'un message d'attente est en cours par exemple)
	killPopUp();
	// puis cration de la nouvelle popup
	_PopUp = new PopUp(this, type, titre, texte, btn1_txt, btn1_evt, btn2_txt, btn2_evt, btn3_txt, btn3_evt);
}

void Lineo::killPopUp(void)
{
	delete(_PopUp);
	_PopUp=NULL;
}

// cette fonction ne se quiterra que lorsque la PopUp ne sera plus affiche
// ATTENTION : CETTE FONCTION EST BLOQUANTE, c'est le but mem de la fonction
// mais il vaut mieux l'utiliser dans un thread....
// dans la pratique elle est utile avant l'appel d'un event, car il serait
//perdu si une popup est affiche
void Lineo::waitUntilPopUpIsNotNull(void){
	while(_PopUp != NULL){
		SDL_Delay(100);
	}
}
