//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#include "Piece.h"
#include "Lineo.h"

// on initialise toutes les donnes suivant la pice qui est indique
Piece::Piece(Lineo* app_lineo, Uint8 forme)
{
	_AppLineo = app_lineo;
    static Uint32 indice=0;

    _Forme = forme;

    _Selected = false;
    _OnScreen = false;

    string color;
    if(_Forme & P_FORME_COLOR_1){
        color = _AppLineo->_Config->get("couleur1");
    }else{
        color = _AppLineo->_Config->get("couleur2");
    }

    _SurfaceName = (string)(DATA_PATH)+"game/pieces/"+color+"_3d.png";
    _SurfaceNameCont = (string)(DATA_PATH)+"game/pieces/contour_3d.png";

    // grce  l'indice qui est unique (car static) le nom du sprite le sera galement
    _SpriteName = "piece_" + to_string(indice++);
    _SpriteNameCont = "piece_contour_" + to_string(indice++);

    // on formate plusieurs paramtres pour les 16 pices
    if(_Forme & P_FORME_CARRE && _Forme & P_FORME_PETITE && _Forme & P_FORME_PLEINE){
        _OffsetX=20;
        _OffsetY=41;
        _RecSrc.x=14;
        _RecSrc.y=10;
        _RecSrc.w=41;
        _RecSrc.h=52;
        _OffsetXCont=26;
        _OffsetYCont=47;
        _RecSrcCont.x=8;
        _RecSrcCont.y=4;
        _RecSrcCont.w=53;
        _RecSrcCont.h=63;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_PETITE && _Forme & P_FORME_CREUSE){
        _OffsetX=20;
        _OffsetY=41;
        _RecSrc.x=76;
        _RecSrc.y=10;
        _RecSrc.w=41;
        _RecSrc.h=52;
        _OffsetXCont=26;
        _OffsetYCont=47;
        _RecSrcCont.x=70;
        _RecSrcCont.y=4;
        _RecSrcCont.w=52;
        _RecSrcCont.h=63;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_PETITE && _Forme & P_FORME_PLEINE){
        _OffsetX=24;
        _OffsetY=39;
        _RecSrc.x=133;
        _RecSrc.y=12;
        _RecSrc.w=48;
        _RecSrc.h=48;
        _OffsetXCont=29;
        _OffsetYCont=45;
        _RecSrcCont.x=128;
        _RecSrcCont.y=6;
        _RecSrcCont.w=59;
        _RecSrcCont.h=59;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_PETITE && _Forme & P_FORME_CREUSE){
        _OffsetX=24;
        _OffsetY=39;
        _RecSrc.x=195;
        _RecSrc.y=12;
        _RecSrc.w=47;
        _RecSrc.h=48;
        _OffsetXCont=30;
        _OffsetYCont=45;
        _RecSrcCont.x=189;
        _RecSrcCont.y=6;
        _RecSrcCont.w=59;
        _RecSrcCont.h=59;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_GRANDE && _Forme & P_FORME_PLEINE){
        _OffsetX=20;
        _OffsetY=70;
        _RecSrc.x=14;
        _RecSrc.y=74;
        _RecSrc.w=41;
        _RecSrc.h=81;
        _OffsetXCont=26;
        _OffsetYCont=75;
        _RecSrcCont.x=8;
        _RecSrcCont.y=69;
        _RecSrcCont.w=53;
        _RecSrcCont.h=92;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_GRANDE && _Forme & P_FORME_CREUSE){
        _OffsetX=20;
        _OffsetY=70;
        _RecSrc.x=76;
        _RecSrc.y=74;
        _RecSrc.w=41;
        _RecSrc.h=81;
        _OffsetXCont=26;
        _OffsetYCont=75;
        _RecSrcCont.x=70;
        _RecSrcCont.y=69;
        _RecSrcCont.w=52;
        _RecSrcCont.h=92;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_GRANDE && _Forme & P_FORME_PLEINE){
        _OffsetX=24;
        _OffsetY=67;
        _RecSrc.x=133;
        _RecSrc.y=77;
        _RecSrc.w=48;
        _RecSrc.h=77;
        _OffsetXCont=29;
        _OffsetYCont=73;
        _RecSrcCont.x=128;
        _RecSrcCont.y=71;
        _RecSrcCont.w=59;
        _RecSrcCont.h=88;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_GRANDE && _Forme & P_FORME_CREUSE){
        _OffsetX=24;
        _OffsetY=67;
        _RecSrc.x=195;
        _RecSrc.y=77;
        _RecSrc.w=47;
        _RecSrc.h=76;
        _OffsetXCont=30;
        _OffsetYCont=73;
        _RecSrcCont.x=189;
        _RecSrcCont.y=71;
        _RecSrcCont.w=59;
        _RecSrcCont.h=88;
    }
}

Piece::~Piece()
{
	freeScreen();
}

// dessine la pices dans la classe screen
void Piece::drawScreen(Uint16 x, Uint16 y, Uint32 z_index)
{
    _PosPointX = x;
    _PosPointY = y;

    _ZIndex = z_index;

    _RecDes.x = _PosPointX - _OffsetX;
    _RecDes.y = _PosPointY - _OffsetY;

    _RecDes.w = _RecSrc.w;
    _RecDes.h = _RecSrc.h;

    _AppLineo->_Screen->addSprite(_SpriteName,_SurfaceName,_ZIndex);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_RecSrc,_RecDes);

    _OnScreen = true;
}

// repositionne la pice  l'cran
void Piece::updateScreen(Uint16 x, Uint16 y, Uint32 z_index)
{
    _PosPointX = x;
    _PosPointY = y;

    _ZIndex = z_index;

    if(_OnScreen){
        // on rcupre la position actuelle
        SDL_Rect old_rect=_RecDes;
        // on calcul la nouvelle position
        _RecDes.x = _PosPointX - _OffsetX;
        _RecDes.y = _PosPointY - _OffsetY;
        // on calcul l'offset entre les deux positions
        Sint32 offset_x = _RecDes.x - old_rect.x;
        Sint32 offset_y = _RecDes.y - old_rect.y;
        // on repositionne dans Screen d'aprs l'offset
        _AppLineo->_Screen->movSpriteAllImages(_SpriteName,offset_x,offset_y);
        // on repositionne galement le z_index
        _AppLineo->_Screen->setSpriteZIndex(_SpriteName,_ZIndex);
        // si la pice est slectionner on update galement la slection
        if(_Selected){
            // on repositionne dans Screen d'aprs l'offset
            _AppLineo->_Screen->movSpriteAllImages(_SpriteNameCont,offset_x,offset_y);
            // on repositionne galement le z_index
            _AppLineo->_Screen->setSpriteZIndex(_SpriteNameCont,_ZIndex-1);
        }
    }
}

// supprime la pices de la classe screen
void Piece::freeScreen(void)
{
    _AppLineo->_Screen->delSprite(_SpriteName);
    _AppLineo->_Screen->delSprite(_SpriteNameCont);

    _OnScreen = false;
}

// dtecte si on survol la pice (true) ou non (false)
bool Piece::testSurvol(Uint16 x_mouse, Uint16 y_mouse)
{
    if(_OnScreen){
        // on dcale les coordonnes de la souris sur l'angle en haut  gauche de la pice
        Sint16 x = x_mouse - _RecDes.x;
        Sint16 y = y_mouse - _RecDes.y;
        // on vrifie d'abord qu'on est bien sur la surface
        if(x>=0 && x<_RecSrc.w && y>=0 && y<_RecSrc.h){
            // puis qu'on est bien sur la pice en vrifiant la transparence qui doit tre  255 (opaque) pour tre sr
            if(_AppLineo->_Screen->getSpriteSurfacePixelAlpha(_SpriteName,x+_RecSrc.x,y+_RecSrc.y)==SDL_ALPHA_OPAQUE){
                return true;
            }
        }
    }
    return false;
}

// place ou enlve un contour autour de la pice
void Piece::setSelected(bool selected)
{
    if(_Selected!=selected){
        _Selected=selected;
        if(_Selected==true && _OnScreen==true){
            _RecDesCont.x = _PosPointX - _OffsetXCont;
            _RecDesCont.y = _PosPointY - _OffsetYCont;

            _RecDesCont.w = _RecSrcCont.w;
            _RecDesCont.h = _RecSrcCont.h;

            _AppLineo->_Screen->addSprite(_SpriteNameCont,_SurfaceNameCont,_ZIndex-1);
            _AppLineo->_Screen->addSpriteImage(_SpriteNameCont,_RecSrcCont,_RecDesCont);
        }else{
            _AppLineo->_Screen->delSprite(_SpriteNameCont);
        }
    }

}
