//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include <vector>
#include <algorithm>

#include "Lineo.h"
#include "Screen.h"

Screen::Screen(Lineo *app_lineo)
{
	_AppLineo = app_lineo;
    _FlagDraw = false;
    _FlagIsDrawing = false;
    _ShowFPS = _AppLineo->_Config->get("show_fps")=="ON" ? true : false ;
}

Screen::~Screen()
{
	SDL_FreeSurface(_SurfaceScreen);
}

// Dessine tous les objets  l'cran
void Screen::draw()
{
    // si rien n'a chang depuis le dernier affichage on ne fait rien
    // ou si par le plus grand des hasard on est dj entrain de faire un draw ??? on pare  toute ventualit pour blinder...
    if(!_FlagDraw || _FlagIsDrawing){
        return;
    }
	// permet de connatre le temps d'excution de la fonction
	Uint32 ticks = SDL_GetTicks();
    // maintenant plus rien ne peut tre fait sur la classe pendant le dessin  l'cran
    _FlagIsDrawing = true;
    // on vrifi si au moins 1 ZIndex a t modifi
    // si ou le flag==true et donc on refait un tri
    if(_flagZIndex){
        // alors on tri dans l'ordre croissant
        triZIndex();
        // puis on raz le flag
        _flagZIndex = false;
    }
    // Effacer tout l'cran
    SDL_FillRect(_SurfaceScreen,NULL,0);
    // Blit (copie) sur la surface de l'cran (_Surface)
    int sprites_size=_Sprites.size();
    for(int i=0;i<sprites_size;i++){
        _Sprites[i].blitOnSurface(_Surfaces[_Sprites[i].getIndexSurface()].getSurface(),_SurfaceScreen);
    }
	// On update la surface  l'cran
	SDL_UpdateRect(_SurfaceScreen,0,0,0,0);
	// on repositionne le flag  false vu que la vue vient d'tre redessine
	_FlagDraw = false;
    // on peut  nouveau faire des choses dans la classe Screen
    _FlagIsDrawing = false;
	// ceci n'est pas vraiment un FPS car a aurait peu d'intrt, mais a joue un rle similaire
	//  savoir nous donner le temps que met la fonction draw  s'excuter
	if(_ShowFPS){
		fprintf(stderr,"draw en %3d ms \n", SDL_GetTicks()-ticks);
	}
}

// Ajoute un Sprite  afficher
// Retourne un pointeur sur le sprite cr et ajout dans la liste,
// ==> a permet de dfinir facilement les images du sprite.
// ==> name doit tre unique pour chaque sprite de la liste !!!
// return false si un sprite du mme nom est dj dans la liste
bool Screen::addSprite(string name, string name_surface, Uint32 z_index, bool no_current)
{
    waitWhileIsDrawing();
    // on cr un sprite d'aprs les attributs passs en paramtre
    Sprite sprite_finded(name);
    // on vrifi qu'aucun sprite ne porte dj ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), sprite_finded);
    if (itRecherche != _Sprites.end())
    {
        // on a trouv un sprite qui a dj ce nom
        return false;
    }
    // on ajoute la surface  la liste des surfaces si elle n'existe pas encore
    addSurface(name_surface);
    // on cr un sprite d'aprs les attributs passs en paramtre
    Sprite new_sprite((*findSurface(name_surface)).getIndex(),name);
    new_sprite.setZIndex(z_index);
    if(no_current){
        new_sprite.noCurrent();
    }
    // on ajoute ce sprite  la liste _Sprites
    _Sprites.push_back(new_sprite);
    // on incrmente le nombre de sprite de la surface utilise
    // par le sprite que l'on vient d'ajouter
    _Surfaces[_Sprites.back().getIndexSurface()].addSprite();
    // on met le flag ZIndex  true, car il faudra retrier
    _flagZIndex = true;
    // Tout c'est bien pass
    return true;
}

bool Screen::addSprite(string name, string name_surface, Uint32 z_index)
{
    return addSprite(name, name_surface, z_index, false);
}

// ajoute une image  un sprite
// name : permet de trouver le sprite correspondant
bool Screen::addSpriteImage(string name, SDL_Rect rect_src, SDL_Rect rect_dest)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite alors on lui ajoute l'image
        (*itRecherche).addImage(rect_src,rect_dest);
        // on met le flag Draw  true car il faudra redessin la scne
        _FlagDraw = true;
        return true;
    }
    // on a trouve aucun sprite qui porte ce nom
    return false;
}

// modifie le ZIndex du sprite indiqu par name
bool Screen::setSpriteZIndex(string name, Uint32 z_index)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite alors on lui modifie le ZIndex
        (*itRecherche).setZIndex(z_index);
        // on met le flag ZIndex  true, car il faudra retrier
        _flagZIndex = true;
        // on met le flag Draw  true car il faudra redessin la scne
        _FlagDraw = true;
        return true;
    }

    return false;
}

// modifie le Current du sprite indiqu par name
bool Screen::setSpriteCurrent(string name, Uint16 current)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite alors on lui modifie le current
        (*itRecherche).setCurrent(current);
        // on met le flag Draw  true car il faudra redessiner la scne
        _FlagDraw = true;
        return true;
    }

    return false;
}

// positionne l'image 0 du sprite  (x,y) et repositionne toutes les autres images en consquence
bool Screen::setSpritePosXY(string name, Uint16 new_x, Uint16 new_y)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite
        // on applique cet position  toutes les images
        (*itRecherche).setPosXY(new_x, new_y);
        // on met le flag Draw  true car il faudra redessiner la scne
        _FlagDraw = true;
        return true;
    }

    return false;
}

// repositionne toutes les image d'un sprite d'aprs un offset (position destination)
bool Screen::movSpriteAllImages(string name, Sint32 offset_x, Sint32 offset_y)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite
        // on applique cet offset  toutes les images
        (*itRecherche).movAllImages(offset_x, offset_y);
        // on met le flag Draw  true car il faudra redessiner la scne
        _FlagDraw = true;
        return true;
    }

    return false;
}

// renvois un pointeur sur la surface du sprite
SDL_Surface* Screen::getSpriteSurface(string name)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite
        return _Surfaces[(*itRecherche).getIndexSurface()].getSurface();
    }

    return NULL;
}

// donne la valeur apha du pixel (x,y) de la surface du sprite donn
Uint8 Screen::getSpriteSurfacePixelAlpha(string name, Uint16 x, Uint16 y)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on a trouve le bon sprite
        return _Surfaces[(*itRecherche).getIndexSurface()].getPixelAlpha(x,y);
    }

    return 255;
}

// Supprime un Sprite de la liste
bool Screen::delSprite(string name)
{
    waitWhileIsDrawing();
    Sprite new_sprite(name);
    // on cherche le sprite qui porte ce nom (name)
    vsprite::iterator itRecherche=find(_Sprites.begin(), _Sprites.end(), new_sprite);
    if (itRecherche != _Sprites.end())
    {
        // on dcrmente le nombre de sprite de la surface utilise
        // par le sprite que l'on va supprimer
        _Surfaces[(*itRecherche).getIndexSurface()].delSprite();
        // on a trouv le bon sprite alors on le supprime
        _Sprites.erase(itRecherche);
        // on met le flag Draw  true car il faudra redessin la scne
        _FlagDraw = true;
        return true;
    }

    return false;
}

// initialise la surface sur laquelle on colera tous les objets pour ensuite l'afficher
// si on ne peut pas afficher n pein cran alors on renvois false afin de ressayer en fentr
bool Screen::setVideoMode(bool full_screen)
{
    if(full_screen){
        _SurfaceScreen = SDL_SetVideoMode(SCREEN_W, SCREEN_H, 0, SDL_FULLSCREEN );
        if (_SurfaceScreen == NULL){
            fprintf(stderr,
                      "\nErreur: Je ne peux pas initialiser le mode vido %dx%d en plein cran.\n"
                      "L'erreur que SDL a renvoy tait:\n"
                      "%s\n\n", SCREEN_W, SCREEN_H, SDL_GetError());
            return false;
        }
    }else{ // SDL_HWSURFACE ==> todo pour tester...
        _SurfaceScreen = SDL_SetVideoMode(SCREEN_W, SCREEN_H, 0, SDL_SWSURFACE | SDL_DOUBLEBUF );
        if (_SurfaceScreen == NULL){
            fprintf(stderr,
                      "\nErreur: Je ne peux pas initialiser le mode vido %dx%d en fentr.\n"
                      "L'erreur que SDL a renvoy tait:\n"
                      "%s\n\n", SCREEN_W, SCREEN_H, SDL_GetError());
            exit(1);
        }
    }

    return true;
}

// tri les sprites d'aprs ZIndex croissant
void Screen::triZIndex()
{
     sort(_Sprites.begin(), _Sprites.end(), SortByZIndex());
}

// ajoute une Surface dans la liste
// >>> Si et seulement si elle n'existe pas dj !!!
// *** IMPORTANT ***
// il ne faut jamais retir une surface de la liste, car pour plus de rapidit
// on pointe directement dans le tableau vector pour ne pas faire une recherche
// et donc si on supprime une surface tout rique d'tre dcal
// C'est pourquoi il est IMPORTANT de laisser la classe Screen, grer elle-mme
// ses surface, ce qui se rsume  en crer si besoins est,  enlever de la
// mmoire l'allocation si elle n'est plus utilise,
// *** ET NE JAMAIS EFFACER UNE SURFACE ***
bool Screen::addSurface(string name)
{
    // on vrifi qu'aucune surface ne porte dj ce nom (name)
    if (findSurface(name) != _Surfaces.end())
    {
        // on a trouv une surface qui a dj ce nom
        return false;
    }
    // on cr et on ajoute une surface  la liste _Surfaces avec le nom (name)
    // _Surfaces.size() correspond  l'index du nouveau vector donc c'est le bon
    // pour la nouvelle classe Surface
    Surface new_surface(name,_Surfaces.size());
    _Surfaces.push_back(new_surface);
    // Tout c'est bien pass
    return true;
}

// rcupre l'itrateur d'une surface de la liste trouve d'aprs le nom (name)
vsurface::iterator Screen::findSurface(string name)
{
    Surface surface_finded(name);
    return find(_Surfaces.begin(), _Surfaces.end(), surface_finded);
}

// boucle qui blocque le programme ou le thread et qui empeche que toute action
// soit effectu dans Screen tant que Screen est entrain de dessiner  l'cran
void Screen::waitWhileIsDrawing(void)
{
	while(_FlagIsDrawing){
        SDL_Delay(10);
	}
}
