//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include <vector>
#include <string>

#include <SDL.h>

#include "Sprite.h"

Sprite::Sprite(Uint32 index_surface, string name)
{
    _Name=name;
    _IndexSurface=index_surface;
    _Current=-1;
    _ZIndex=0;
    _noCurrent = false;
}

// utile pour les finds
Sprite::Sprite(string name)
{
    _Name=name;
}

Sprite::~Sprite()
{
}

// place le current  l'endroit voulu
// ceci permet de choisir l'image voulue
// si > size alors on fait rien
void Sprite::setCurrent(Uint16 x)
{
    if(x<_RectSrc.size()){
        _Current = (int)x;
    }
}

// ajoute une image au sprite
void Sprite::addImage(SDL_Rect rect_src, SDL_Rect rect_dest)
{
    _RectSrc.push_back(rect_src);
	_RectDest.push_back(rect_dest);
	_Current=_RectSrc.size()-1;
}

// positionne l'image 0 du sprite  (x,y) et repositionne toutes les autres images en consquence
void Sprite::setPosXY(Uint16 new_x, Uint16 new_y)
{
    if(_RectDest.size()>0){
		Uint16 old_x = _RectDest[0].x;
		Uint16 old_y = _RectDest[0].y;
		Sint32 offset_x = new_x - old_x;
		Sint32 offset_y = new_y - old_y;
        movAllImages(offset_x,offset_y);
    }
}

// repositionne toutes les images du sprite d'aprs un offset en x et en y
void Sprite::movAllImages(Sint32 offset_x, Sint32 offset_y)
{
    if(_RectDest.size()>0){
        for(Uint32 i=0;i<_RectDest.size();i++){
            _RectDest[i].x+=offset_x;
            _RectDest[i].y+=offset_y;
        }
    }
}


// permet d'indiquer le ZIndex du sprite
void Sprite::setZIndex(Uint32 z)
{
	_ZIndex = z;
}

// Blit on a surface (screen for example)
bool Sprite::blitOnSurface(SDL_Surface* surface_src,SDL_Surface* surface_des)
{
    if(_Current<0){
        // le sprite n'a pas d'image initialise
        return false;
    }

    // si _noCurrent==true alors on liste toutes les images, sinon on affiche que la _Current
    if(_noCurrent){
        for(Uint32 i=0;i<_RectSrc.size();i++){
            if(SDL_BlitSurface(surface_src, &_RectSrc[i], surface_des, &_RectDest[i]) < 0){
                fprintf(stderr,
                    "\nErreur: je ne peux pas excuter le BlitSurface: \n"
                    "L'erreur que SDL a renvoy tait:\n"
                    "%s\n\n",SDL_GetError());
            }
        }
    }else{
        if(SDL_BlitSurface(surface_src, &_RectSrc[_Current], surface_des, &_RectDest[_Current]) < 0){
            fprintf(stderr,
                "\nErreur: je ne peux pas excuter le BlitSurface: \n"
                "L'erreur que SDL a renvoy tait:\n"
                "%s\n\n",SDL_GetError());
        }
    }
    return true;
}
