//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Texte.h"
#include "Lineo.h"

// ce constructeur doit tre utilis quand on compte affich un texte  l'cran
Texte::Texte(Lineo *app_lineo, string font, string texte, char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 pos_x, Uint32 pos_y, Uint32 z_index)
{
	_AppLineo = app_lineo;
    // on remplace les deux caractre "\n" qui peuvent venir du texte d'un fichier par le caractre unique '\n'
    texte = Texte::strReplace("\\n", "\n", texte);
	// on enregistre toutes les variables
	_Font=font;
    _Texte=texte;
    _ZIndex=z_index;
    // on initialise l'identifiant unique
	static Uint32 unique_id_cpt=0;
    _UniqueId = unique_id_cpt++;
    // on dcoupe le texte en phrases selon '\n'
    cutTxtToPhrases();
    // on calcul les coordonne afin de positionner le texte avec prcision
    calculPositions(align_x_y, offset_x, offset_y, pos_x, pos_y);
    // et pour finir on affiche les phrases
    drawPhrases();
}

// ce constructeur permet de rcupr les dimensions largeur et hauteur d'un texte
// sans avoir besoin de l'afficher  l'cran
// on initialise donc un minimum de chose
// ==> CE CONSTRUCTEUR NE PERMET PAS L'AFFICHAGE D'UN TEXTE !!!
Texte::Texte(Lineo *app_lineo, string font, string texte)
{
	_AppLineo = app_lineo;
    // on remplace les deux caractre "\n" qui peuvent venir du texte d'un fichier par le caractre unique '\n'
    texte = Texte::strReplace("\\n", "\n", texte);
	// on enregistre toutes les variables
	_Font=font;
    _Texte=texte;
    // on dcoupe le texte en phrases selon '\n'
    cutTxtToPhrases();
	// puis on calcul et on enregistre la hauteur et la largeur
	// de la totalit des phrases see trouvant dans le texte
	calculDimensions();
}

// !!!! ne doit tre utilis que pour des find !!!!!
Texte::Texte(Uint32 unique_id)
{
    _UniqueId=unique_id;
}

Texte::~Texte()
{
}

// dplace le texte selon un offset
void Texte::mov(Sint32 offset_x, Sint32 offset_y)
{
	// on dplace selon l'offset chacune des phrases
    for(Uint32 i=0;i<_NbPhrases;i++){
		_AppLineo->_VFont->movPhrase(_Font, _Phrases[i].getPhrase(), _Phrases[i].getUniqueId(), offset_x, offset_y);
	}
}

// modifie le z_index
void Texte::setZIndex(Uint32 new_z_index)
{
	// on enregistre le nouveau z_index
	_ZIndex=new_z_index;
	// on modifie le z_index de toutes les phrases
    for(Uint32 i=0;i<_NbPhrases;i++){
		_AppLineo->_VFont->setPhraseZIndex(_Font, _Phrases[i].getPhrase(), _Phrases[i].getUniqueId(), new_z_index);
	}
}

// efface les phrases de l'cran
void Texte::freeScreen(void)
{
	freePhrases();
}

// on coupe le texte en phrase en utilisant le sparateur '\n'
void Texte::cutTxtToPhrases(void)
{
    istringstream iss(_Texte);
    string phrase;
    // on coupe le texte en phrase en utilisant le sparateur '\n'
    while(getline(iss, phrase, '\n')){
	    // on enregistre toutes les phrases dans un vecteur
		Phrase new_phrase(phrase);
        _Phrases.push_back(new_phrase);
    }
    _NbPhrases = _Phrases.size();
}

// on calcul les coordonne afin de positionner le texte avec prcision
void Texte::calculPositions(char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 pos_x, Uint32 pos_y)
{
    Uint16 interligne = 0;
    // on calcul la hauteur totale
	Uint16 h_tot = _NbPhrases * _AppLineo->_VFont->getHeight(_Font) + (_NbPhrases-1)* interligne;
    // on calcul la position en Y si besoin est
    // ceci donne le premier indice de dpart pour les lignes suivantes
    if(align_x_y & TEXTE_Y_CENTER){
        pos_y = ((SCREEN_H - h_tot) / 2);
    }else if(align_x_y & TEXTE_Y_TOP){
        pos_y = 0 + offset_y;
    }else if(align_x_y & TEXTE_Y_BOTTOM){
        pos_y = SCREEN_H - h_tot - offset_y;
    }
    // on initialise la taille totale du texte, a peut servir avec les get...
    _Width=0;
    _Height=0;
    // on fait tous les calculs suivants autant de fois qu'il y a de phrase
    for(Uint32 i=0;i<_NbPhrases;i++){
        string phrase = _Phrases[i].getPhrase();
        // on prpare un rectangle sur la position et
		// les dimensions de la phrase  afficher
        SDL_Rect rect_phrase;
        rect_phrase.x = pos_x;
        rect_phrase.y = pos_y;
        // on calcul la largeur de la phrase
        rect_phrase.w = _AppLineo->_VFont->getPhraseWidth(_Font,phrase.c_str());
        _Width=Max(rect_phrase.w,_Width);
        // on calcul la hauteur de la phrase
        rect_phrase.h = _AppLineo->_VFont->getHeight(_Font);
        _Height=Max(rect_phrase.h,_Height);
        // on recalcul le pos_y pour la prochaine ligne
        pos_y += rect_phrase.h + interligne;
        // on calcul l'alignement en X si besoin est
        if(align_x_y & TEXTE_X_CENTER){
            rect_phrase.x = ((SCREEN_W - rect_phrase.w) / 2);
        }else if(align_x_y & TEXTE_X_LEFT){
            rect_phrase.x = 0 + offset_x;
        }else if(align_x_y & TEXTE_X_RIGHT){
            rect_phrase.x = SCREEN_W - rect_phrase.w - offset_x;
        }
        // on enregistre la position et les dimension de la phrase
		_Phrases[i].setRect(rect_phrase);
	}
}

// affichage des phrases  l'cran
void Texte::drawPhrases(void)
{
	// on parcours la liste des phrases et on les affiche une  une
    for(Uint32 i=0;i<_NbPhrases;i++){
		_AppLineo->_VFont->phraseToImages(_Font, _Phrases[i].getPhrase(), _Phrases[i].getUniqueId(), _Phrases[i].getRect(), _ZIndex);
	}
}

// suppression des phrases de l'cran
void Texte::freePhrases(void)
{
	// on parcours la liste des phrases et on les supprime une  une
    for(Uint32 i=0;i<_NbPhrases;i++){
		_AppLineo->_VFont->delPhrase(_Font, _Phrases[i].getPhrase(), _Phrases[i].getUniqueId());
	}
}

// donne la position d'une chaine dans une autre chaine
Uint32 Texte::strPos(string chaine, string tofind){
    Sint32 pos = chaine.find(tofind);
    if(pos!=-1)
        pos++;
    return pos;
}

// permet de remplacer toutes les occurences d'une chaine par une autre chaine dans une chaine
string Texte::strReplace(string avant, string apres, string chaine) {
    Sint32 pos = strPos(chaine, avant);
    while(pos!=-1) {
        chaine.replace(pos-1, avant.length(), apres);
        pos = Texte::strPos(chaine, avant);
    }
    return chaine;
}

// on calcul les coordonne afin de positionner le texte avec prcision
void Texte::calculDimensions(void)
{
    // on initialise la taille totale du texte, a peut servir avec les get...
    _Width=0;
    _Height=0;
    // on fait tous les calculs suivants autant de fois qu'il y a de phrase
    for(Uint32 i=0;i<_NbPhrases;i++){
        string phrase = _Phrases[i].getPhrase();
        _Width=Max((Uint16)_AppLineo->_VFont->getPhraseWidth(_Font,phrase.c_str()),_Width);
        _Height+=(Uint16)_AppLineo->_VFont->getHeight(_Font);
	}
}
