//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VFont.h"
#include "Lineo.h"

VFont::VFont(Lineo *app_lineo)
{
	_AppLineo = app_lineo;
}

VFont::~VFont()
{
}

// ajoute une font  la liste, si elle n'existe pas dj
bool VFont::add(string name_font, string name_surface, string ext)
{
    // on vrifi qu'aucun Font ne porte dj ce nom (name)
    if (findFont(name_font) != _Fonts.end())
    {
        // on a trouv un Font qui a dj ce nom, a ne sert  rien de le rajouter
        return false;
    }
    // on cr et on ajoute un Font  la liste _Fonts avec le nom (name)
    Font new_font(_AppLineo, name_font, name_surface, ext);
    _Fonts.push_back(new_font);
    // Tout c'est bien pass
    return true;
}

// donne la largeur d'une phrase en pixel
Uint32 VFont::getPhraseWidth(string name_font, string phrase)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on retourne donc la largeur de la phrase
        return (*itRecherche).getPhraseWidth(phrase);
    }
    // on a pas trouv de Font initialise avec ce nom
    return 0;
}

// donne la hauteur du Texte d'un font en pixel
Uint32 VFont::getHeight(string name_font)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on retourne donc la hauteur du texte
        return (*itRecherche).getHeight();
    }
    // on a pas trouv de Font initialise avec ce nom
    return 0;
}

// transforme une phrase en autant d'image qu'il y a de lettres
void VFont::phraseToImages(string name_font, string phrase, Uint32 id_phrase, SDL_Rect rect, Uint32 z_index)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on retourne donc la hauteur du texte
        (*itRecherche).phraseToImages(phrase, id_phrase, rect, z_index);
    }
}

// dplace un texte selon un offset
void VFont::movPhrase(string name_font, string phrase, Uint32 id_phrase, Sint32 offset_x, Sint32 offset_y)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on modifie le z_index de la phrase
        (*itRecherche).movPhrase(phrase, id_phrase, offset_x, offset_y);
    }
    // on a pas trouv de Font initialise avec ce nom
}

// modifie le z_index d'une phrase
void VFont::setPhraseZIndex(string name_font, string phrase, Uint32 id_phrase, Uint32 new_z_index)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on modifie le z_index de la phrase
        (*itRecherche).setPhraseZIndex(phrase, id_phrase, new_z_index);
    }
    // on a pas trouv de Font initialise avec ce nom
}

// supprime une phrase et renvoi le z_index qui tait au texte
void VFont::delPhrase(string name_font, string phrase, Uint32 id_phrase)
{
    // on cherche le bon Font
    vfont::iterator itRecherche=findFont(name_font);
    if (itRecherche != _Fonts.end())
    {
        // on a bien trouv un Font qui a ce nom
        // on supprime donc la phrase de l'cran
        (*itRecherche).delPhrase(phrase, id_phrase);
    }
    // on a pas trouv de Font initialise avec ce nom
}

// rcupre l'itrateur d'un Font de la liste trouve d'aprs le nom (name)
vfont::iterator VFont::findFont(string name)
{
    Font font_finded(name);
    return find(_Fonts.begin(), _Fonts.end(), font_finded);
}
