//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueChoixLangue.h"
#include "Lineo.h"

VueChoixLangue::VueChoixLangue(Lineo* app_lineo)
{
	_AppLineo = app_lineo;
	// on initialise la liste des langues
	_ListeLang.clear();
    _ListeLang["fr"]    ="Franais";
    _ListeLang["en"]    ="English";
    _ListeLang["de"]    ="Deutsch";
//    _ListeLang["es"]    ="Espaol";
    _ListeLang["se"]    ="Svenska";
    _ListeLang["fr_br"] ="Brezhoneg";
	// on initialise les images des drapeaux, en listant toutes les langues
	// les dimensions sont les mme  chaque fois on ne les calcul donc qu'une fois
    SDL_Rect RecDes, RecSrc;
    RecSrc.x=0; RecSrc.y=0;
    RecSrc.w=260; RecSrc.h=180;
	RecDes.x = RecDes.y = 0;
    RecDes.w = RecSrc.w; RecDes.h = RecSrc.h;
    for(mapSSit it=_ListeLang.begin();it!=_ListeLang.end();it++){
	    _AppLineo->_Screen->addSprite((*it).first,(string)(DATA_PATH)+"lang/flags/flag_"+(*it).first+".png",Z_MIN);
    	_AppLineo->_Screen->addSpriteImage((*it).first,RecSrc,RecDes);
    }
    // on initialise le fond d'cran
    _FondEcran = new FondEcran(_AppLineo, FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
    // on initialise le menu
    _Menu = new Menu(_AppLineo, _AppLineo->_Langue->get("Choix de la langue"), true, EVENT_CREATE_VUE_OPTION,300);
    _ListeLangIter=getItVectLang(_AppLineo->_Config->get("langue"));
    _NoItemLang = _Menu->addItem((*_ListeLangIter).second,EVENT_CHOIX_LANGUE,MENU_TYPE_LISTE,MENU_EVENT_NO_DATA,(*_ListeLangIter).first,RecSrc.w,RecSrc.h);
    _Menu->draw();
}

VueChoixLangue::~VueChoixLangue()
{
    // on efface les drapeaux
    for(mapSSit it=_ListeLang.begin();it!=_ListeLang.end();it++){
        _AppLineo->_Screen->delSprite((*it).first);
    }
    // on libre les pointeurs des classes
	delete(_FondEcran);
    delete(_Menu);
}

// permet d'aller  l'itrateur suivant, ou retourner au premier si le dernier est atteind
mapSSit VueChoixLangue::listeLangNext(mapSSit current_iter)
{
	current_iter++;
	if(current_iter==_ListeLang.end()){
        current_iter = _ListeLang.begin();
	}
	return current_iter;
}

// permet d'aller  l'itrateur prcdent, ou retourner au dernier si le premier est atteind
mapSSit VueChoixLangue::listeLangPrev(mapSSit current_iter)
{
	if(current_iter==_ListeLang.begin()){
        current_iter = _ListeLang.end();
	}
	current_iter--;
	return current_iter;
}

// rcupre l'itrateur d'un drapeau dans la liste
// si on trouve pas le drapeau on renvois le premier
// la valeur retourne est de type std::vector<std::string, std::allocator<std::string>>
mapSSit VueChoixLangue::getItVectLang(string langue)
{
    mapSSit itRecherche=_ListeLang.begin();
    while(itRecherche!=_ListeLang.end()){
	    if((*itRecherche).first == langue){
	        return itRecherche;
	    }
        itRecherche++;
    }
	return _ListeLang.begin();
}


// gre les vnements de la vue
void VueChoixLangue::processEvents(SDL_Event *event)
{
    // on s'occupe de notre vue en premier
    switch (event->type) {
        case SDL_USEREVENT:
            switch (event->user.code)
            {
                case EVENT_CHOIX_LANGUE:{
                    Uint16 fleche = *(Uint16*)event->user.data1;
                    if(fleche==MENU_FLECHE_R){
                 		_ListeLangIter = listeLangPrev(_ListeLangIter);
					}else if(fleche==MENU_FLECHE_L){
                 		_ListeLangIter = listeLangNext(_ListeLangIter);
					}
					_AppLineo->_Config->set("langue",(*_ListeLangIter).first);
              		_Menu->updateItemListe(_NoItemLang,(*_ListeLangIter).second,(*_ListeLangIter).first);
					_AppLineo->_Langue->Lire();
                }break;
            }
        break;
   	}
   	// puis du menu
    _Menu->processEvents(event);
}
