//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueGame.h"
#include "Lineo.h"

VueGame::VueGame(Lineo *app_lineo, Uint8 mode_jeu)
{
	_AppLineo = app_lineo;
	_ModeJeu = mode_jeu;
    // on vrifie si on doit afficher la vue de dessus ou non
    _DisplayVueDessus = (_AppLineo->_Config->get("vue_dessus")=="ON" ? true : false );
    // on initialise tous les pointeurs de classes  NULL
	_GamePlateau=NULL;
	_LaDonne=NULL;
	_MenuFin=NULL;
	_FondEcran=NULL;
	_VueDessus=NULL;
	_PlayerCurrent=NULL;
	_PlayerG=NULL;
	_PlayerD=NULL;
	_QuiCommence=PAS_DE_VALEUR;
	// on initialise le thread  null au cas o on a pas le temps de l'utiliser...
	_Thread     = NULL;
	_StopThread = false;
	// on initialise l'id de la classe  0, utile  la suppression
	_PhraseLineoId = 0;
    // on initialise les deux joueurs
    switch(_ModeJeu){
        case CHOIX_JOUER_MODE_1_JOUEUR:
            _PlayerG = new Joueur(_AppLineo, JOUEUR_TYPE_HUMAIN, _AppLineo->_Config->get("joueurG"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(_AppLineo, JOUEUR_TYPE_IA, "I.A.", JOUEUR_DROITE);
        break;
        case CHOIX_JOUER_MODE_2_JOUEURS:
            _PlayerG = new Joueur(_AppLineo, JOUEUR_TYPE_HUMAIN, _AppLineo->_Config->get("joueurG"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(_AppLineo, JOUEUR_TYPE_HUMAIN, _AppLineo->_Config->get("joueurD"), JOUEUR_DROITE);
        break;
#ifndef Nokia770
        case CHOIX_JOUER_MODE_RESEAU:
            _PlayerG = new Joueur(_AppLineo, JOUEUR_TYPE_HUMAIN, _AppLineo->_Config->get("login"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(_AppLineo, JOUEUR_TYPE_RESO, JOUEUR_DROITE);
        break;
#endif
    }
    // on initialise le fond d'cran
    _FondEcran = new FondEcran(_AppLineo, FD_ECR_VIERGE|FD_ECR_NO_LOGO_NATSIMHAN|FD_ECR_LINEO_SMALL|FD_ECR_NO_PIECES_3D);
    // on affiche les noms des joueurs s'ils sont dfinits
    if(_PlayerD->getNomIsSet()){
        _TxtPlayerGId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
    }else{
        _TxtPlayerGId=0;
    }
    if(_PlayerD->getNomIsSet()){
        _TxtPlayerDId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
    }else{
        _TxtPlayerDId=0;
    }
    // on initialise et on dessine la vue du plateau avec ses pices
    _GamePlateau = new GamePlateau(_AppLineo);
    // on initialise et on dessine la vue de dessus si elle est demande
    if(_DisplayVueDessus){
        _VueDessus = new VueDessus(_AppLineo);
    }
#ifndef Nokia770
    // maintenant on peut dmarrer la partie si on est pas en mode rseau
    if(_ModeJeu!=CHOIX_JOUER_MODE_RESEAU){
        _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_START);
    }
#endif
}

VueGame::~VueGame()
{
    // on demande au thread de s'arrter
    _StopThread = true;
    // on attend que le thread se termine tout seul de faon  viter tout problme
    if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
    // puis la phrase gagn ou perdu ou galit
    if(_PhraseLineoId>0){
        _AppLineo->_VTexte->del(_PhraseLineoId);
    }
    // puis les noms des joueurs
    _AppLineo->_VTexte->del(_TxtPlayerGId);
    _AppLineo->_VTexte->del(_TxtPlayerDId);
    // on supprime toutes les classes
    delete(_GamePlateau);
    delete(_LaDonne);
    delete(_MenuFin);
    delete(_FondEcran);
    delete(_VueDessus);
    delete(_PlayerG);
	delete(_PlayerD);
}

// gre les vnements de la vue
void VueGame::processEvents(SDL_Event *event)
{
    // on s'occupe de notre vue en premier
    switch (event->type) {
        case SDL_USEREVENT:
            // ce flag permettra  l'IA de donner la pice
            // sans qu'un autre vnement arrivant ventuellement
            // rapidement derrire ne supprime l'venement
            if(event->user.code==EVENT_GAME_PIECE_WAIT){
                _FlagEventGamePieceWait=true;
            }
            // on s'occupe de traiter l'venement
            switch (event->user.code) {
                case EVENT_GAME_PARTIE_START:{
                    // on arrte de jouer la musique pour pas nerver pendant la partie
                    Musique::getInstance()->decrechendo(3000);
                    // on initialise la donne
                    _LaDonne = new LaDonne();
                    // on initialise la prise en charge des carrs si mode impossible et pas en rseau
                    if(_AppLineo->_Config->get("niveau")=="impossible" && _ModeJeu!=CHOIX_JOUER_MODE_RESEAU){
                        _LaDonne->setCarresPossibles();
                    }
                    // on met toutes les pices autour du plateau
                    for(Uint8 i=0;i<16;i++){
                        _GamePlateau->addAutourPiece(i, _LaDonne->getPieceLibre(i));
                    }
                    // on dcide quel joueur commencera la partie
                    _QuiCommence = _PlayerD->getQuiCommence();
                    // on definit alors le joueur courrant, celui qui est entrain de jouer
                    _PlayerCurrent = (_QuiCommence == 0 ? _PlayerG : _PlayerD);
                    // on affiche les noms des joueurs s'ils ne sont pas encore affichs
                    if(_TxtPlayerGId==0){
                        _TxtPlayerGId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    if(_TxtPlayerDId==0){
                        _TxtPlayerDId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on indique visuellement quel joueur est entrain de jouer
                    if(_PlayerCurrent->getNom()==_PlayerG->getNom()){
                        // on efface d'abord le nom
                        _AppLineo->_VTexte->del(_TxtPlayerGId);
                        // puis on affiche avec la modif des <<<
                        _TxtPlayerGId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerG->getNom()+FLECHE_G,TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }else{
                        // on efface d'abord le nom
                        _AppLineo->_VTexte->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des >>>
                        _TxtPlayerDId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,FLECHE_D+_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on dit bien que le jeu n'est pas finit (flag pour les threads)
                    _FlagEndGame=false;
                    // puis on attend que le premier joueur choisisse la premire pice
                    _AppLineo->_Events->addEvent(EVENT_GAME_FIRST_PIECE_WAIT);
                }break;
                case EVENT_GAME_FIRST_PIECE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses vnements
                        // en le laissant faire tout seul
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(true);
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien  l'humain
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(false);
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                        // on attend que le thread se termine tout seul de faon  viter tout problme
                        if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
                        //on lance un thread comme a on ne bloque pas l'action du joueur Humain sur le jeu
                        _Thread = SDL_CreateThread(this->staticJoueurFirstThread, this);
                        if ( _Thread == NULL ) {
                            fprintf(stderr, "\nImpossible de crer le thread this->staticJoueurFirstThread\n%s\n\n", SDL_GetError());
                            return;
                        }
                    }
                break;
                case EVENT_GAME_PIECE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses vnements
                        // en le laissant faire tout seul
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(true);
#ifndef Nokia770
                        // afin de signaler qu'on est toujours actif en attendant que l'humain choisisse la pice on ping le serveur en mode rseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU) {
                            _AppLineo->_Events->addEvent(EVENT_GAME_PING);
                        }
#endif
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien  l'humain
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(false);
                    }
                break;
                case EVENT_GAME_PIECE_SELECTED:{
                    // on rcupre la pice choisie
                    Uint8 *autour_i = (Uint8*)event->user.data1;
#ifndef Nokia770
                    if( (_ModeJeu==CHOIX_JOUER_MODE_RESEAU) && (_PlayerCurrent->isHuman()) ) {
                        _PlayerD->pieceSelected(*autour_i);
                    }
#endif
                    // On change la donne
                    _LaDonne->choisirPiece(*autour_i);
                    // on prcise  GamePlateau qu'il ne doit plus attendre d'vnements de choix de pice
                    _GamePlateau->setPieceWait(false);
                    // on place la pice devant le plateau  l'adversaire du joueur en court
                    _GamePlateau->addChoixPiece(*autour_i,(_PlayerCurrent->getPlace()==JOUEUR_GAUCHE));
                    // on supprime la pice autour du plateau
                    _GamePlateau->delAutourPiece((Uint8)(*autour_i));
                    // on change alors de joueur sauf si on est un humain en mode rseau
                    // auquel cas il faut attendre que l'infos soit envoye avant de faire le changement
                    if( (_ModeJeu!=CHOIX_JOUER_MODE_RESEAU) || (!_PlayerCurrent->isHuman()) ) {
                        _AppLineo->_Events->addEvent(EVENT_GAME_CHANGE_PLAYER);
                    }
                }break;
                case EVENT_GAME_CHANGE_PLAYER:{
                    // on change le joueur actuel
                    if(_PlayerCurrent->getPlace() == JOUEUR_GAUCHE){
                        _PlayerG = _PlayerCurrent;
                        _PlayerCurrent = _PlayerD;
                        // on efface d'abord les noms
                        _AppLineo->_VTexte->del(_TxtPlayerGId);
                        _AppLineo->_VTexte->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des >>> sur le bon
                        _TxtPlayerGId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                        _TxtPlayerDId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,FLECHE_D+_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }else{
                        _PlayerD = _PlayerCurrent;
                        _PlayerCurrent = _PlayerG;
                        // on efface d'abord le nom
                        _AppLineo->_VTexte->del(_TxtPlayerGId);
                        _AppLineo->_VTexte->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des <<<
                        _TxtPlayerDId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                        _TxtPlayerGId = _AppLineo->_VTexte->add(VGAME_FONT_JOUEUR,_PlayerG->getNom()+FLECHE_G,TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on attend que le nouveau joueur place la pice en choisissant un cercle
                    if(_PlayerCurrent->isHuman()){
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
                    }else{
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                    }
                    _AppLineo->_Events->addEvent(EVENT_GAME_CERCLE_WAIT);
                }break;
                case EVENT_GAME_CERCLE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses vnements
                        // en le laissant faire tout seul
                        _GamePlateau->setPieceWait(false);
                        _GamePlateau->setCercleWait(true);
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
#ifndef Nokia770
                        // afin de signaler qu'on est toujours actif en attendant que l'humain choisisse le cercle on ping le serveur en mode rseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU) {
                            _AppLineo->_Events->addEvent(EVENT_GAME_PING);
                        }
#endif
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien  l'humain
                        _GamePlateau->setPieceWait(false);
                        _GamePlateau->setCercleWait(false);
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                        // on attend que le thread se termine tout seul de faon  viter tout problme
                        if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
                        //on lance un thread comme a on ne bloque pas l'action du joueur Humain sur le jeu
                        _Thread = SDL_CreateThread(this->staticJoueurThread, this);
                        if ( _Thread == NULL ) {
                            fprintf(stderr, "\nImpossible de crer le thread this->staticJoueurThread\n%s\n\n", SDL_GetError());
                            return;
                        }
                    }
                break;
                case EVENT_GAME_CERCLE_SELECTED:{
                    string reso_partie="";
                    // on rcupre la position du cercle choisi
                    Uint8 i = *(Uint8*)event->user.data1;
                    Uint8 j = *(Uint8*)event->user.data2;
                    // on indique  GamePlateau qu'il ne faut plus couter les vnements des cercles
                    _GamePlateau->setCercleWait(false);
                    // on ajoute la pice choisie sur le cercle choisi
                    _GamePlateau->addPlateauPiece(i,j,_LaDonne->getPieceChoisie());
                    // on l'ajoute galement dans la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->addPlateauPiece(i,j,_LaDonne->getPieceChoisie());
                    }
                    // on supprime la pice choisie vu que maintenant elle est sur le plateau
                    _GamePlateau->delChoixPiece(GP_CHOIX_LEFT);
                    _GamePlateau->delChoixPiece(GP_CHOIX_RIGHT);
                    // on change la donne
                    _LaDonne->placerPiece(i,j);
                    // on test s'il y a un Lineo
                    if(_LaDonne->isLineo()){
                        _FlagEndGame=true;
                        // si on joue en rseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                            reso_partie = "win";
                        }
                        // on arrte donc le jeu et on dit qui a gagn si on est pas en rseau
                        // si c'est un humain qui joue contre autre chose qu'un humain
                        // et que c'est l'autre chose qui a gagn
                        // alors on affiche perdu
                        // sinon gagn dans tous les autres cas
                        else{
                            if((_PlayerG->isHuman() || _PlayerD->isHuman())
                                && !_PlayerCurrent->isHuman()){
                                _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_LOST);
                            }else{
                                _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_WIN);
                            }
                        }
                    }else if(_LaDonne->isEgalite()){
                        _FlagEndGame=true;
                        // si on joue en rseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                            reso_partie = "nulle";
                        }
                        // on arrte le jeu car il ne reste plus de pice et pourtant il n'y a pas de Lineo, mais seulement si on est pas en rseau
                        else{
                            _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_NULLE);
                        }
                    }else{
                        _FlagEndGame=false;
                        // on continu la partie et on attend que le mme joueur choisisse une pice pour l'autre
                        // sauf si on est un humain en mode rseau
                        // auquel cas il faut attendre que l'infos soit envoye avant de continuer
                        if( (_ModeJeu!=CHOIX_JOUER_MODE_RESEAU) || (!_PlayerCurrent->isHuman()) ) {
                            _AppLineo->_Events->addEvent(EVENT_GAME_PIECE_WAIT);
                        }
                    }
#ifndef Nokia770
                    // maintenant on donne au serve la position du cercle choisit si c'est l'humain qui a jou en mode rseau
                    // on en profite pour indiquer si la partie est termine ou non
                    if((_ModeJeu == CHOIX_JOUER_MODE_RESEAU) && (_PlayerCurrent->isHuman())) {
                        // selon que la partie soit finie ou non on agit diffremment
                        if(_FlagEndGame){
                            _PlayerD->cercleSelected(i,j,reso_partie);
                        }else{
                            _PlayerD->cercleSelected(i,j);
                        }
                    }
                    // si c'est le rseau qui a jou alors on confirme que la partie est termine au besoin
                    else if((_ModeJeu == CHOIX_JOUER_MODE_RESEAU) && (!_PlayerCurrent->isHuman()) && _FlagEndGame) {
                        _PlayerD->confirmePartieEnd();
                    }
#endif
                }break;
                case EVENT_GAME_PARTIE_WIN:
                    // on redonne la main  l'humain
                    _GamePlateau->setHumanPlay(true);
                    // si c'est deux humain qui jouent on dit le nom du gagnant
                    if(_PlayerG->isHuman() && _PlayerD->isHuman()){
                        _PhraseLineo = _PlayerCurrent->getNom()+" "+_AppLineo->_Langue->get("a gagne !");
                    }else{
                        _PhraseLineo = _AppLineo->_Langue->get("Vous avez gagne !");
                    }
                    // on surligne les pices et les cercles du Lineo
                    _GamePlateau->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    // on surligne les pices de la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    }
                    // on indique que la partie est finie
                    _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_LOST:
                    // on redonne la main  l'humain
                    _GamePlateau->setHumanPlay(true);
                    _PhraseLineo = _AppLineo->_Langue->get("Vous avez perdu ...");
                    // on surligne les pices et les cercles du Lineo
                    _GamePlateau->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    // on surligne les pices de la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    }
                    // on indique que la partie est finie
                    _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_NULLE:
                    // on redonne la main  l'humain
                    _GamePlateau->setHumanPlay(true);
                    _PhraseLineo = _AppLineo->_Langue->get("Egalite !");
                    // on indique que la partie est finie
                    _AppLineo->_Events->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_END:
                    // on affiche la phrase perdu, gagn ou galit
                    _PhraseLineoId = _AppLineo->_VTexte->add(FONT_58_WHITE,_PhraseLineo,TEXTE_X_CENTER|TEXTE_Y_CENTER,Z_VUE_GAME);
                    // on cr le mini menu en bas pour pouvoir rejouer ou quitter
                    _MenuFin = new Menu(_AppLineo, MENU_SANS_TITRE, false,0);
                    switch(_ModeJeu){
				        case CHOIX_JOUER_MODE_1_JOUEUR:
		                    _MenuFin->addItem(_AppLineo->_Langue->get("Nouvelle partie"),EVENT_CREATE_VUE_GAME,MENU_TYPE_LIEN,CHOIX_JOUER_MODE_1_JOUEUR);
				        break;
				        case CHOIX_JOUER_MODE_2_JOUEURS:
		                    _MenuFin->addItem(_AppLineo->_Langue->get("Nouvelle partie"),EVENT_CREATE_VUE_GAME,MENU_TYPE_LIEN,CHOIX_JOUER_MODE_2_JOUEURS);
						break;
#ifndef Nokia770
						case CHOIX_JOUER_MODE_RESEAU:
                            _MenuFin->addItem(_AppLineo->_Langue->get("Refaire une partie"),EVENT_GAME_NEW_PARTIE);
		                    _MenuFin->addItem(_AppLineo->_Langue->get("Choix d'un nouvel adversaire"),EVENT_CREATE_VUE_CHOIXADVERSAIRE);
						break;
#endif
				    }
                    _MenuFin->addItem(_AppLineo->_Langue->get("Retour au menu"),EVENT_CREATE_VUE_CHOIX_JOUER);
                    _MenuFin->setAlignBottom(60);
                    _MenuFin->draw();
#ifndef Nokia770
                    // on lance le thread d'attente d'une demande de nouvelle partie en mode reseau
                    if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                        _PlayerD->newPartieWait();
                    }
#endif
                    // on recommence  jouer de la musique
                    Musique::getInstance()->play();
                break;
            }
        break;

        case SDL_KEYDOWN:
            if(event->key.keysym.sym==SDLK_ESCAPE){
				// on affiche une popup et on demande la confirmation de quitter la partie en cours
			    _AppLineo->drawPopUp (	POPUP_TYPE_WARNING,
										_AppLineo->_Langue->get("Quitter la partie(Titre)"),
										_AppLineo->_Langue->get("Quitter la partie(Question)"),
										_AppLineo->_Langue->get("Oui"),EVENT_CREATE_VUE_CHOIX_JOUER,
										_AppLineo->_Langue->get("Non"),EVENT_GAME_CONTINUER_PARTIE );
            }else if(event->key.keysym.sym==SDLK_F2){
                // on affiche ou non la vue de dessus suivant l'tat prcdent
                if(_DisplayVueDessus){
                    removeVueDessus();
                }else{
                    showVueDessus();
                }
            }
        break;
   	}
    // puis ceux du menu s'il existe et si on est pas entrain de faire tourner le plateau...
   	if(_MenuFin!=NULL  && _AppLineo->_Curseur->getCurseur()!=CUR_TOURNE){
       	_MenuFin->processEvents(event);
    }
    // puis ceux du plateau et de ses objets seulement si le menu de fin n'existe pas
    // OU si le menu de fin existe et que le curseur n'est pas une main
    if(_GamePlateau!=NULL && _MenuFin==NULL || (_MenuFin!=NULL && _AppLineo->_Curseur->getCurseur()!=CUR_MAIN)){
   	    _GamePlateau->processEvents(event);
    }
    // puis ceux des joueurs si le menu de fin n'existe pas
    if(_PlayerG!=NULL) _PlayerG->processEvents(event);
    if(_PlayerD!=NULL) _PlayerD->processEvents(event);
}

// >>> ceci est le premier coup donc on demande juste une piece !!!
// thread qui tourne a cote pour faire les calculs du joueur
// et ce sans bloquer le programme (affichage, tourner le plateau...)
int VueGame::joueurFirstThread(void *data)
{
    // on recupere l'instance de la classe VueGame
    VueGame *vue_game = (VueGame *)data;
    // on rcupre l'emplacement du cercle o poser la pice
    vue_game->_PlayerCurrent->pieceFirstWait(vue_game,*vue_game->_LaDonne);
    // c'est fini on quitte le thread
    return 0;
}

// thread qui tourne a cote pour faire les calculs du joueur
// et ce sans bloquer le programme (affichage, tourner le plateau...)
int VueGame::joueurThread(void *data)
{
    static Uint32 numero=0;
    numero++;
    // on recupere l'instance de la classe VueGame
    VueGame *vue_game = (VueGame *)data;
    // on met le flag  false il sera mis  true quand l'venement sera envoy
    vue_game->_FlagEventGamePieceWait = false;
    // on rcupre l'emplacement du cercle o poser la pice
    vue_game->_PlayerCurrent->cercleWait(vue_game,*vue_game->_LaDonne);
    // on attend qu'on nous demande de donner la piece
    while(!vue_game->_FlagEventGamePieceWait
    && !vue_game->_FlagEndGame
    && !vue_game->_StopThread)
    {
        // ce delay permet de ne pas monopoliser le processeur pour ceci
        SDL_Delay(200);
    }
    // on donne la piece si la partie n'est pas finie et que le thread ne doit pas s'arreter
    if(!vue_game->_FlagEndGame && !vue_game->_StopThread){
        // on donne la piece
        vue_game->_PlayerCurrent->pieceWait(vue_game,*vue_game->_LaDonne);
    }
    // c'est fini on quitte le thread
    return 0;
}

// affiche la vue de dessus en mettant  jour toutes les pices du plateau
void VueGame::showVueDessus(void){
    // on ajuste le paramtre de config et le flag bool
    _DisplayVueDessus = true;
    _AppLineo->_Config->set("vue_dessus","ON");
    // on initialise la vue
    _VueDessus = new VueDessus(_AppLineo);
    // puis on recherche les pices du plateau pour les afficher sur la vue
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            if(_LaDonne->getPlateau(i,j)!=P_NO_FORME) {
                _VueDessus->addPlateauPiece(i,j,_LaDonne->getPlateau(i,j));
            }
        }
    }
    // si il y a un aurto il faut afficher son contour
    // on surligne les pices de la vue de dessus
    if(_LaDonne->isLineo()){
        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
    }
}

// efface et enlve compltement la vue de dessus et tout ce qui s'y trouve
void VueGame::removeVueDessus(void){
    // on ajuste le paramtre de config et le flag bool
    _DisplayVueDessus = false;
    _AppLineo->_Config->set("vue_dessus","OFF");
    // on supprime la classe de la vue
    delete(_VueDessus);
}
