//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueSousOption.h"
#include "Lineo.h"

VueSousOption::VueSousOption(Lineo* app_lineo, Uint16 parametre)
{
	_AppLineo = app_lineo;
	_Parametre = parametre;
	// on initialise le fond d'cran
	_FondEcran = new FondEcran(_AppLineo, FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
	// On affiche le _Menu qui a t selectionn
	switch (_Parametre){
		case EVENT_OPTION_JOUEUR:
            _NomJoueurG = _AppLineo->_Config->get("joueurG");
            _NomJoueurD = _AppLineo->_Config->get("joueurD");
			_Menu = new Menu(_AppLineo, _AppLineo->_Langue->get("Nom des joueurs"),true,EVENT_CREATE_VUE_OPTION,350);
			_NoItemJoueurG = _Menu->addItem(_AppLineo->_Langue->get("Joueur gauche"),EVENT_JOUEUR_G,&_NomJoueurG);
			_NoItemJoueurD = _Menu->addItem(_AppLineo->_Langue->get("Joueur droite"),EVENT_JOUEUR_D,&_NomJoueurD);
			_Menu->draw();
		break;

		case EVENT_OPTION_MUSIQUE:{
			_Menu = new Menu(_AppLineo, _AppLineo->_Langue->get("Avec ou sans Musique"), true, EVENT_CREATE_VUE_OPTION);
			_NoItemMusiqueOn = _Menu->addItem(_AppLineo->_Langue->get("Musique On"),EVENT_MUSIQUE_ON,MENU_TYPE_RADIO);
			_NoItemMusiqueOff = _Menu->addItem(_AppLineo->_Langue->get("Musique Off"),EVENT_MUSIQUE_OFF,MENU_TYPE_RADIO);
			_Menu->draw();
            string config = _AppLineo->_Config->get("musique");
            if(config == "ON")
            {
                _Menu->setItemOn(_NoItemMusiqueOn);
            }
            else if(config == "OFF")
            {
                _Menu->setItemOn(_NoItemMusiqueOff);
            }
		}break;

		case EVENT_OPTION_VUE_DESSUS:{
			_Menu = new Menu(_AppLineo, _AppLineo->_Langue->get("Vue de dessus"), true, EVENT_CREATE_VUE_OPTION);
			_NoItemDessusOn = _Menu->addItem(_AppLineo->_Langue->get("Oui"),EVENT_VUEDESSUS_ON,MENU_TYPE_RADIO);
			_NoItemDessusOff = _Menu->addItem(_AppLineo->_Langue->get("Non"),EVENT_VUEDESSUS_OFF,MENU_TYPE_RADIO);
			_Menu->draw();
            string config = _AppLineo->_Config->get("vue_dessus");
            if(config == "ON")
            {
                _Menu->setItemOn(_NoItemDessusOn);
            }
            else if(config == "OFF")
            {
                _Menu->setItemOn(_NoItemDessusOff);
            }
		}break;
	}
}

VueSousOption::~VueSousOption()
{
    // on libre les pointeurs des classes
	delete(_FondEcran);
    delete(_Menu);
}

// gre les vnements de la vue
void VueSousOption::processEvents(SDL_Event *event)
{
    switch (_Parametre)
    {
       case EVENT_OPTION_JOUEUR:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_JOUEUR_G :{
                    _AppLineo->_Config->set("joueurG", _NomJoueurG);
                }break;
                case EVENT_JOUEUR_D :{
                    _AppLineo->_Config->set("joueurD", _NomJoueurD);
                }break;
            }
       }break;

       case EVENT_OPTION_MUSIQUE:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_MUSIQUE_ON :{
                    _Menu->setItemOn(_NoItemMusiqueOn);
                    _AppLineo->_Config->set("musique","ON");
                    // on commence  jouer de la musique
                    Musique::getInstance()->init();
                    Musique::getInstance()->play();
                }break;
                case EVENT_MUSIQUE_OFF:{
                    _Menu->setItemOn(_NoItemMusiqueOff);
                    _AppLineo->_Config->set("musique","OFF");
                    // on arrte de jouer la musique pour pas nerver pendant la partie
                    Musique::getInstance()->stop();
                    Musique::kill();
                }break;
            }
       }break;
       case EVENT_OPTION_VUE_DESSUS:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_VUEDESSUS_ON :{
                    _Menu->setItemOn(_NoItemDessusOn);
                    _AppLineo->_Config->set("vue_dessus","ON");
                }break;
                case EVENT_VUEDESSUS_OFF:{
                    _Menu->setItemOn(_NoItemDessusOff);
                    _AppLineo->_Config->set("vue_dessus","OFF");
                }break;
            }
       }break;
    }
    _Menu->processEvents(event);
}


